/* ************************************************************************** */

#include "extract_dialog.h"


static void callback_extract_dialog_destroy( GtkWidget *widget,
											 gpointer   user_data )
{
  ST_ExtractDialog_cbData *cbData;
  
  /* User_data to extract dialog callback data */
  cbData = (ST_ExtractDialog_cbData *)user_data;
  
  /* Just let's go */
  gtk_widget_destroy(widget);
  
}

static void callback_extract_dialog_cancel( GtkButton *button,
											gpointer user_data)
{
  ST_ExtractDialog_cbData *cbData;
  
  /* User_data to extract dialog callback data */
 cbData = (ST_ExtractDialog_cbData *)user_data;
  
  /* Destroy main extract window */
  gtk_widget_destroy(cbData->window);
}

static void callback_extract_dialog_extract( GtkButton *button,
											 gpointer user_data)
{
  ST_ExtractDialog_cbData *cbData;
  gboolean extract_all_files;
  gchar *dir, *file;
  GList *selected_files = NULL;

  /* User_data to extract dialog callback data */
  cbData = (ST_ExtractDialog_cbData *)user_data;
   
  /* Save actual options as default ones */
  lintar_set_extract_overwrite(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cbData->overwrite)));
  lintar_set_extract_skip_new(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cbData->skip)));
  lintar_set_extract_recursive(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cbData->recursive)));
  
  /* Get some options */
  extract_all_files = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cbData->all));
  dir = g_strdup(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(cbData->combo)->entry)));
  
  /* Get Current Archive */
  file = lintar_get_current_filename();

  /* Create the list of files to extract */
  if (!extract_all_files) {
	if (lintar_get_current_n_selected_files()>0) {
	  selected_files = lintar_get_current_selected_files(lintar_get_extract_recursive());
	}
  }

  /* Destroy main extract window */
  gtk_widget_destroy(cbData->window);

  while (gtk_events_pending()) gtk_main_iteration();

  /* Extract archive */
  extract_archive( file, dir, lintar_get_extract_overwrite(),
				   lintar_get_extract_skip_new(), selected_files);
  

  /* Free memory for selected */
  if (selected_files!=NULL) {
	g_list_foreach( selected_files, gchar_list_free, NULL);
	g_list_free(selected_files);
  }

}


static void callback_extract_dialog_browse( GtkButton *button,
											gpointer user_data)
{
  GtkWidget *dialog, *combo;
  gchar *str;

  /* Get callback data */
  combo = (GtkWidget *)user_data;

  /* Open file selection dialog */
  dialog = gtk_file_selection_new(_("Select Extract Directory"));

  /* Display that dialog */
  switch ( gtk_dialog_run(GTK_DIALOG(dialog)) ) {
  case GTK_RESPONSE_OK:
  case GTK_RESPONSE_ACCEPT:
	/* Get name selected on the dir dialog */
	str = g_strdup(gtk_file_selection_get_filename( GTK_FILE_SELECTION(dialog)));
	
	/* Copy dir to combo box */
	gtk_entry_set_text( GTK_ENTRY(GTK_COMBO(combo)->entry), str);
	break;
  default:
	break;
  }

  /* Destroy it */
  gtk_widget_destroy (dialog);

}



GtkWidget *create_extract_dialog( GtkWidget *parent, 
								  guint8 type)
{
  GtkWidget *window, *table;
  GtkWidget *extract_frame, *extract_table, *extract_browse, *extract_combo;
  GtkWidget *files_frame, *files_table, *files_radio1, *files_radio2;
  GtkWidget *options_frame, *options_table, *options_overwrite, *options_skip;
  GtkWidget *options_recursive;
  GtkWidget *button_extract, *button_cancel, *button_box;
  gchar current_dir[LINTAR_MAX_LENGTH];

  static ST_ExtractDialog_cbData cbData;

  /* 
   * Create Main window
   */
  
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title( GTK_WINDOW (window), _("Extract"));
  gtk_signal_connect( GTK_OBJECT (window), "destroy",
					  GTK_SIGNAL_FUNC(callback_extract_dialog_destroy), 
					  (gpointer)&cbData);
  
  gtk_container_set_border_width( GTK_CONTAINER(window), 5);
  
  /* Create table for storing all widgets */
  table = gtk_table_new (3, 2, FALSE);
  gtk_container_add( GTK_CONTAINER(window), table);
  
  
  /* 
   * Create Frame "Extract to"
   */

  extract_frame = gtk_frame_new(_("Extract Archive to"));
  extract_table = gtk_table_new(1, 2, FALSE);
  gtk_container_set_border_width( GTK_CONTAINER(extract_table), 5);  
  gtk_container_add( GTK_CONTAINER(extract_frame), extract_table);

  /* Create text box */
  /* FIXME: Add default strings to combobox */
  extract_combo = gtk_combo_new();
  getcwd(current_dir, LINTAR_MAX_LENGTH-1);
  gtk_entry_set_text( GTK_ENTRY(GTK_COMBO(extract_combo)->entry), current_dir);

  gtk_table_attach( GTK_TABLE(extract_table), extract_combo, 
					/* X direction */          /* Y direction */
					0, 1,                      0, 1,
					GTK_EXPAND | GTK_FILL,     GTK_EXPAND | GTK_FILL,
					0,                         2);
  
  /* Create Browse Button */
  extract_browse = gtk_button_new_with_mnemonic(_("_Browse..."));
  g_signal_connect( GTK_OBJECT(extract_browse), "clicked", 
					G_CALLBACK(callback_extract_dialog_browse), 
					(gpointer)(extract_combo));
  gtk_table_attach( GTK_TABLE(extract_table), extract_browse, 
					/* X direction */          /* Y direction */
					1, 2,                      0, 1,
					0,                         0,
					2,                         2);
  
  
  /* Attach extract to table to frame */
  gtk_table_attach( GTK_TABLE(table), extract_frame, 
					/* X direction */          /* Y direction */
					0, 2,                      0, 1,
					GTK_EXPAND | GTK_FILL,     GTK_EXPAND | GTK_FILL,
					0,                         5);
  
  
  /* 
   * Create Frame "Files"
   */

  files_frame = gtk_frame_new(_("Files"));
  files_table = gtk_table_new(2, 1, FALSE);
  gtk_container_set_border_width( GTK_CONTAINER(files_table), 5);  
  gtk_container_add( GTK_CONTAINER(files_frame), files_table);
  
  files_radio1 = gtk_radio_button_new_with_mnemonic( NULL, _("_Selected files"));
  files_radio2 = gtk_radio_button_new_with_mnemonic_from_widget( GTK_RADIO_BUTTON(files_radio1),
																 _("_All files"));

  /* Active "All" if no files selected on tree */
  if (lintar_get_current_n_selected_files()>0) {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(files_radio1), TRUE);
  } else {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(files_radio2), TRUE);
	gtk_widget_set_sensitive(files_radio1, FALSE);
  }

  gtk_table_attach( GTK_TABLE(files_table), files_radio1, 
					/* X direction */          /* Y direction */
					0, 1,                      0, 1,
					GTK_EXPAND | GTK_FILL,     GTK_EXPAND | GTK_FILL,
					0,                         0);
  gtk_table_attach( GTK_TABLE(files_table), files_radio2, 
					/* X direction */          /* Y direction */
					0, 1,                      1, 2,
					GTK_EXPAND | GTK_FILL,     GTK_EXPAND | GTK_FILL,
					0,                         0);

  gtk_table_attach( GTK_TABLE(table), files_frame, 
					/* X direction */          /* Y direction */
					0, 1,                      1, 2,
					GTK_EXPAND | GTK_FILL,     0,
					2,                         0);


  /* 
   * Create Frame "Options"
   */

  options_frame = gtk_frame_new(_("Options"));
  options_table  = gtk_table_new(3, 1, FALSE);
  gtk_container_set_border_width( GTK_CONTAINER(options_table), 14);  

  options_overwrite = gtk_check_button_new_with_mnemonic(_("_Overwrite existing files"));
  options_skip = gtk_check_button_new_with_mnemonic(_("S_kip new files"));
  options_recursive = gtk_check_button_new_with_mnemonic(_("_Recursive selected subdirectories"));

  /* Select default options */
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(options_overwrite), 
								lintar_get_extract_overwrite());
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(options_skip), 
								lintar_get_extract_skip_new());
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(options_recursive), 
								lintar_get_extract_recursive());
  

  /* Shadow some options in some cases */
  switch(type) {
  case LINTAR_TYPE_TAR:
  case LINTAR_TYPE_TAR_Z:
  case LINTAR_TYPE_TAR_GZ:
  case LINTAR_TYPE_TAR_BZ2:
	gtk_widget_set_sensitive( options_skip, FALSE);
	break;
  case LINTAR_TYPE_ZIP:
  case LINTAR_TYPE_UNKNOWN:
  default:
	break;
  }
  
  gtk_table_attach( GTK_TABLE(options_table), options_overwrite, 
					/* X direction */          /* Y direction */
					0, 1,                      0, 1,
					GTK_EXPAND | GTK_FILL,     0,
					0,                         0);
  gtk_table_attach( GTK_TABLE(options_table), options_skip, 
					/* X direction */          /* Y direction */
					0, 1,                      1, 2,
					GTK_EXPAND | GTK_FILL,     0,
					0,                         0);
  gtk_table_attach( GTK_TABLE(options_table), options_recursive, 
					/* X direction */          /* Y direction */
					0, 1,                      2, 3,
					GTK_EXPAND | GTK_FILL,     0,
					0,                         0);

  gtk_table_attach( GTK_TABLE(table), options_table, 
					/* X direction */          /* Y direction */
					1, 2,                      1, 2,
					GTK_EXPAND | GTK_FILL,     0,
					0,                         0);

  
  /* 
   * Callback data
   */
  cbData.window = window;
  cbData.combo = extract_combo;
  cbData.selected = files_radio1;
  cbData.all = files_radio2;
  cbData.overwrite = options_overwrite;
  cbData.skip = options_skip;
  cbData.recursive = options_recursive;


  /*
   * Add Extract and Cancel buttons
   */

  button_box = gtk_hbutton_box_new();
  button_extract = gtk_button_new_from_stock(GTK_STOCK_OK);
  button_cancel  = gtk_button_new_from_stock(GTK_STOCK_CANCEL);

  g_signal_connect( GTK_OBJECT(button_extract), "clicked", 
					G_CALLBACK(callback_extract_dialog_extract),
					(gpointer)(&cbData));
  g_signal_connect( GTK_OBJECT(button_cancel), "clicked", 
					G_CALLBACK(callback_extract_dialog_cancel),
					(gpointer)(&cbData));

  gtk_container_set_border_width( GTK_CONTAINER(button_box), 5);
  gtk_button_box_set_layout( GTK_BUTTON_BOX(button_box), GTK_BUTTONBOX_END);
  gtk_box_set_spacing( GTK_BOX(button_box), 10);
 
  gtk_container_add( GTK_CONTAINER(button_box), button_cancel);
  gtk_container_add( GTK_CONTAINER(button_box), button_extract);

  gtk_table_attach( GTK_TABLE(table), button_box, 
					/* X direction */          /* Y direction */
					1, 2,                      2, 3,
					GTK_EXPAND | GTK_FILL,     GTK_EXPAND | GTK_FILL,
					0,                         0);

  /* Select OK button as default */
  gtk_widget_grab_focus(button_extract);

  /* Set extract window modal */
  gtk_window_set_modal( GTK_WINDOW(window), TRUE);
  gtk_window_set_transient_for( GTK_WINDOW(window), GTK_WINDOW(parent));

  return(window);

}








