/* ************************************************************************** */

#include "general.h"

void lintar_destroy_all(void)
{
  /* Save all unsaved data */
  lintar_data_save();

  /* Free memory */
  lintar_data_free();
  
}


void lintar_callback_action(gchar *action)
{
  GtkWidget *dialog;
  gchar *str;


  if (g_ascii_strcasecmp(action,"<menu>/Help/About")==0) {
	
	dialog = gtk_message_dialog_new( GTK_WINDOW(lintar_get_main_window()),
									 GTK_DIALOG_DESTROY_WITH_PARENT,
									 GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,
									 lintar_get_long_about());
	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);


  } else if ( (g_ascii_strcasecmp(action,"<menu>/File/Quit")==0)
			  || (g_ascii_strcasecmp(action,"<toolbar>/Quit")==0) ) {
	
	lintar_destroy_all();
	gtk_main_quit();
	
  } else if ( (g_ascii_strcasecmp(action,"<menu>/File/Close")==0)
			  || (g_ascii_strcasecmp(action,"<toolbar>/Close")==0) ) {
	
	/* Close actual file opened */
	lintar_close_archive( gtk_notebook_current_page(GTK_NOTEBOOK(lintar_get_notebook())) );
	
  } else if ( (g_ascii_strcasecmp(action,"<menu>/File/Open")==0)
			  || (g_ascii_strcasecmp(action,"<toolbar>/Open")==0) ) {
	
	/* Open file selection dialog */
	dialog = gtk_file_selection_new(_("Open Archive"));
	
	/* Display that dialog */
	switch ( gtk_dialog_run(GTK_DIALOG(dialog)) ) {
	case GTK_RESPONSE_OK:
	case GTK_RESPONSE_ACCEPT:

	  str = g_strdup( gtk_file_selection_get_filename(GTK_FILE_SELECTION(dialog)) );

	  /* Destroy dialog */
	  gtk_widget_destroy(dialog);

	  /* Make sure all gtk is ok */
	  while (gtk_events_pending()) gtk_main_iteration();

	  /* Open archive from filename */
	  open_archive(str);
	  
	  /* Free things */
	  g_free(str);
	  break;
	default:
	  /* Destroy dialog */
	  gtk_widget_destroy(dialog);
	  break;
	}
	
  } else if ( (g_ascii_strcasecmp(action,"<menu>/File/Extract")==0) 
			  || (g_ascii_strcasecmp(action,"<popup>/Extract")==0)
			  || (g_ascii_strcasecmp(action,"<toolbar>/Extract")==0) ) {
	
	/* Open file selection dialog */
	dialog = create_extract_dialog( lintar_get_main_window(), 
									lintar_get_current_filetype());
	
	/* Display that dialog */
	gtk_widget_show_all(dialog);

  } else if ( (g_ascii_strcasecmp(action,"<menu>/File/View")==0)
			  || (g_ascii_strcasecmp(action,"<popup>/View")==0)
			  || (g_ascii_strcasecmp(action,"<toolbar>/View")==0) ) {

	//view_selected_files(lintar_get_extract_recursive());
	view_selected_files(FALSE);

  } else if ( (g_ascii_strcasecmp(action,"<menu>/Options/Preferences")==0) ) {

	/* Open file prefs dialog */
	dialog = create_prefs_dialog( lintar_get_main_window() );
	
	/* Display that dialog */
	gtk_widget_show_all(dialog);
	

  } else {
	
	dialog = gtk_message_dialog_new( GTK_WINDOW(lintar_get_main_window()), 
									 GTK_DIALOG_DESTROY_WITH_PARENT,
									 GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,
									 "Callback action \"%s\" unknown!", action);
	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);

  }


}

GdkPixbuf *lintar_get_pixbuf_from_file( GtkWidget *window, 
										guint8 type,
										gchar *fullname)
{
  GdkPixbuf *pix = NULL, *pix2=NULL;
  const gchar *icon=NULL, *mime=NULL;
  gchar *path=NULL, *str=NULL, *mime2=NULL, *pl=NULL;
  gint i;

  switch (type) {
  case LINTAR_TYPE_FOLDER:
	pix = gtk_widget_render_icon( window, GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU, NULL );
	break;
  case LINTAR_TYPE_LINK:
	pix = gtk_widget_render_icon( window, GTK_STOCK_JUMP_TO, GTK_ICON_SIZE_MENU, NULL);
	break;
  default:

	/* Try to get mime icon */
	mime = (const gchar *)gnome_vfs_mime_type_from_name( (const char*)fullname);
	if (mime!=NULL) {
	  icon = (const gchar *)gnome_vfs_mime_get_icon(mime);
	  if (icon!=NULL) {
		path = gnome_vfs_icon_path_from_filename(icon);
	  }
	}
	 
	if (path==NULL) {
	  if (mime!=NULL) {
		/* Try to find filename from mime type and pixmap folders */
		mime2 = g_strdup(mime);
		if (index(mime2,'/')!=NULL) { (index(mime2,'/'))[0] = '-'; }
		
		i = 0;
		pl = lintar_data.pixmaps_lst[i++];
		while (pl!=NULL) {
		  str = g_strdup_printf(pl, mime2);
		  if (g_file_test( str, G_FILE_TEST_EXISTS)) {
			path = str; pl = NULL;
		  }
		  pl = lintar_data.pixmaps_lst[i++];
		}
		
		g_free(mime2);
	  }
	}

	if (path==NULL) {
	  pix = gtk_widget_render_icon( window, GTK_STOCK_NEW, GTK_ICON_SIZE_MENU, NULL);
	} else {
	  pix = gdk_pixbuf_new_from_file(path, NULL);
	  if (pix==NULL) {
		pix = gtk_widget_render_icon( window, GTK_STOCK_NEW, GTK_ICON_SIZE_MENU, NULL);
	  } else {
		pix2 = pix;
		pix = gdk_pixbuf_scale_simple(pix2, 16, 16, GDK_INTERP_BILINEAR);
		gdk_pixbuf_unref(pix2);
	  }
	  g_free(path);
	}
	break;
  }
  
  return(pix);

}


void lintar_set_widgets_sensitive(gboolean b)
{
  
  /* Set some toobal and menu widget sensitive */ 
  gtk_widget_set_sensitive( lintar_get_toolbar_view(), b);
  gtk_widget_set_sensitive( lintar_get_toolbar_extract(), b);
  gtk_widget_set_sensitive( lintar_get_toolbar_close(), b);
  gtk_widget_set_sensitive(gtk_item_factory_get_widget_by_action(lintar_get_item_factory(),
																 3), b);
  gtk_widget_set_sensitive(gtk_item_factory_get_widget_by_action(lintar_get_item_factory(),
																 4), b);
  gtk_widget_set_sensitive(gtk_item_factory_get_widget_by_action(lintar_get_item_factory(), 
																 6), b);
  
  /* Display something else on label */
  if (b==TRUE) {
	gtk_widget_hide( lintar_get_label() );
	gtk_widget_show (lintar_get_notebook() );
  } else {
	gtk_widget_hide( lintar_get_notebook() );
	gtk_widget_show (lintar_get_label() );
	lintar_push_statusbar(lintar_get_short_about());
  }

  while (gtk_events_pending()) gtk_main_iteration();
}

static void callback_notebook_close( GtkWidget *button,
									 gpointer data)
{
  
  /* Close actual file opened */
  lintar_close_archive( gtk_notebook_page_num( GTK_NOTEBOOK(lintar_get_notebook()), 
											   (GtkWidget *)data));

}

void lintar_notebook_add_page( GtkWidget *sw, gchar *file)
{
  gchar *str;
  GtkWidget *label, *hbox, *close_image, *button;

  /* Horizontal box to pack label + close button */
  hbox  = gtk_hbox_new(FALSE, 0);

  /* Create label and pack it into hbox */
  str   = g_path_get_basename(file);
  label = gtk_label_new(str);
  gtk_misc_set_alignment( GTK_MISC(label), 0.00, 0.5);
  gtk_misc_set_padding( GTK_MISC(label), 4, 0);
  gtk_box_pack_start( GTK_BOX(hbox), label, TRUE, TRUE, 0);

  /* Create close button and pack it into hbox */
  close_image = gtk_image_new_from_stock( GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
  gtk_widget_show(close_image);
  button = gtk_button_new();
  gtk_button_set_relief( GTK_BUTTON(button), GTK_RELIEF_NONE);
  gtk_container_set_border_width( GTK_CONTAINER(button), 0);
  gtk_container_add( GTK_CONTAINER(button), close_image);
  gtk_widget_set_size_request( button, 15, 15);
  gtk_box_pack_start( GTK_BOX(hbox), button, FALSE, FALSE, 0);
  gtk_signal_connect( GTK_OBJECT(button), "clicked", 
					  GTK_SIGNAL_FUNC(callback_notebook_close), sw);

  gtk_widget_show_all(hbox);

  /* Create new notebookpage */
  gtk_notebook_append_page( GTK_NOTEBOOK( lintar_get_notebook() ), sw, hbox);

  /* free allocated memory */
  g_free(str);

  /* make sure all is displayed */
  while (gtk_events_pending()) gtk_main_iteration();
}

void lintar_select_notebook_current_page( void)
{
  gtk_notebook_set_current_page( GTK_NOTEBOOK(lintar_get_notebook() ), 
								 lintar_get_current_archive() );
}


void lintar_get_full_filename( gchar *output, gchar *shortname, gchar **fullname)
{
  gchar *pc;
  gint i, l;
  
  pc = strstr(output, shortname); 
  i = pc-output; l = 0;
  while (output[i]!='\n') { l++; i++; }
  l++;
  
  /* Now l is the size of the full filename */
  (*fullname) = (gchar*)g_malloc( (gulong)sizeof(gchar)*l);
  
  /* Copy name to string */
  i = pc-output; l = 0;
  while (output[i]!='\n') {
	(*fullname)[l++] = output[i++];
  }
  (*fullname)[l] = '\0';

}

void lintar_extract_symbolic_link( gchar *output, gchar *filename, gchar **link, guint8 *type)
{

  gchar *pc;
  
  pc = strstr(filename,"->");
  if (pc!=NULL) {
	/* there is a link, copy link info */
	(*link) = g_strdup((gchar*)(pc+3)); 
	/* and remove last -> */
	filename[pc - filename - 1] = '\0'; /* count extra space before "->" */
	(*type) = LINTAR_TYPE_LINK;	  
  } else {
	(*link) = g_strdup("");
  }

}

void lintar_remove_directory( gchar *permissions, gchar *name, guint8 *type)
{

  gint l;

  l = strlen(name);

  if (permissions[0]=='d') {
	if (name[l-1]!='/') {
	  /* This shouldn't happen :-? */
	  g_print( _("Permissions say file is a directory but there is *NOT* a trailing '/' in the filename."));
	  exit(-1);
	} else {
	  name[l-1]='\0';
	  (*type) = LINTAR_TYPE_FOLDER;
	}
  }

}



gboolean casecmp_ext(gchar *file, gchar *ext)
{
  gint len_ext, len_file;
  gchar *p;

  len_file = strlen(file);
  len_ext  = strlen(ext);

  if (len_file>=len_ext) {
	
	p = file + len_file - len_ext;
	
	if (g_ascii_strcasecmp(p,ext)==0) {
	  return(TRUE);
	}

  }

  return(FALSE);

}


guint8 lintar_get_compress_file_type( gchar *file) 
{
  const gchar *mime=NULL;


  mime = (const gchar *)gnome_vfs_get_file_mime_type( (const char*)file, NULL, FALSE);
  
  if (mime!=NULL) {
	
	if ( g_ascii_strcasecmp(mime,"application/zip")==0 ) {

	  return(LINTAR_TYPE_ZIP);

	} else if ( g_ascii_strcasecmp(mime,"application/x-bzip")==0 ) {
	  
	  if ( casecmp_ext(file, ".tar.bz2") || casecmp_ext(file, ".tar.bz") ) {
		return(LINTAR_TYPE_TAR_BZ2);
	  } else {
		return(LINTAR_TYPE_BZ2);
	  }

	} else if ( g_ascii_strcasecmp(mime,"application/x-gzip")==0 ) {

	  return(LINTAR_TYPE_GZ);

	} else if ( g_ascii_strcasecmp(mime,"application/x-compressed-tar")==0 ) {

	  return(LINTAR_TYPE_TAR_GZ);

	} else if ( g_ascii_strcasecmp(mime,"application/x-compress")==0 ) {

	  if ( casecmp_ext(file, ".tar.Z") || casecmp_ext(file, ".tarZ") ) {
		return(LINTAR_TYPE_TAR_Z);
	  } else {
		return(LINTAR_TYPE_Z);
	  }

	} else if ( g_ascii_strcasecmp(mime,"application/x-tar")==0 ) {

	  return(LINTAR_TYPE_TAR);

	} else {

	  return(LINTAR_TYPE_UNKNOWN);

	}

  } else {

	return(LINTAR_TYPE_UNKNOWN);

  }

}

gpointer thread_wait(gpointer data)
{
  //gint i = 0;
  //gchar *str;

  //gdk_threads_enter();
  
  gtk_progress_bar_set_pulse_step( GTK_PROGRESS_BAR(lintar_get_progressbar()), 0.05);

  while(lintar_get_thread()) {

	//str = g_strdup_printf("THREAD_WAIT(%d)", i++);
	//fprintf( stderr, str);

	//lintar_push_statusbar(str);

	//lintar_mutex_lock();
	gtk_progress_bar_pulse( GTK_PROGRESS_BAR(lintar_get_progressbar()) );

	gtk_widget_show(lintar_get_progressbar());
	gtk_widget_show_all(lintar_get_progressbar());
	gtk_widget_show_now(lintar_get_progressbar());
	while (gtk_events_pending()) gtk_main_iteration();

	gtk_widget_show( lintar_get_main_window() );
	gtk_widget_show_all( lintar_get_main_window() );
	gtk_widget_show_now( lintar_get_main_window() );
	while (gtk_events_pending()) gtk_main_iteration();
	//lintar_mutex_unlock();
	
	//g_free(str);
	usleep(10000);

  }

  //gdk_threads_leave();

  g_thread_exit(NULL);

  return(NULL);

}

gboolean thread_g_spawn_sync( const gchar *working_directory,
							  gchar **argv,
							  gchar **envp,
							  GSpawnFlags flags,
							  GSpawnChildSetupFunc child_setup,
							  gpointer user_data,
							  gchar **standard_output,
							  gchar **standard_error,
							  gint *exit_status,
							  GError **error)
{
  GThread *thread;
  gboolean spawn;

  lintar_set_thread(TRUE);
  thread = g_thread_create( thread_wait, NULL, TRUE, NULL);
  
  spawn = g_spawn_sync( working_directory, argv, envp, flags, child_setup, user_data, 
						standard_output, standard_error, exit_status, error);

  lintar_set_thread(FALSE);

  /* Wait for other thread to finish */
  g_thread_join(thread);


  return(spawn);
}


static void callback_popup_menu( gpointer  callback_data,
								 guint     callback_action,
								 GtkWidget *widget)
{
  gchar *str;
  
  switch (callback_action) {
  case  0: str = g_strdup("<popup>/View");            break;
  case  1: str = g_strdup("<popup>/sep0");            break;
  case  2: str = g_strdup("<popup>/Extract");         break;
  default: str = g_strdup("<popup>/callback_action"); break;
  }
  
  /* Call the action to do */
  lintar_callback_action(str);
  
  g_free(str);
  
}

GtkItemFactory *lintar_create_popup_menu(void)
{
  GtkItemFactory*      popup_item;
  GtkItemFactoryEntry* popup_items;  

  /* Create the menubar */
  //accel_group = gtk_accel_group_new();
  popup_item = gtk_item_factory_new( GTK_TYPE_MENU, "<main>", NULL);

  /* Set up item factory to go away with the window */
  g_object_ref(popup_item);
  gtk_object_sink(GTK_OBJECT(popup_item));
  g_object_set_data_full( G_OBJECT(lintar_get_main_window()), "<main>", popup_item,
						  (GDestroyNotify)g_object_unref);

  /* create menu items */
  popup_items = (GtkItemFactoryEntry *)g_malloc(1*sizeof(GtkItemFactoryEntry));
  
  if (lintar_get_current_n_selected_files()>0) {
	if (lintar_get_current_n_selected_files()==1) {
	  popup_items->path            = _("/View");
	} else {
	  popup_items->path            = _("/View All"); 
	}
	popup_items->accelerator     = NULL;
	popup_items->callback        = callback_popup_menu;
	popup_items->callback_action = 0;
	popup_items->item_type       = "<StockItem>";;
	popup_items->extra_data      = GTK_STOCK_EXECUTE;
	gtk_item_factory_create_item( popup_item,popup_items, NULL, 1);

	/* Separator */
	popup_items->path            = "/sep0";
	popup_items->accelerator     = NULL;
	popup_items->callback        = NULL;
	popup_items->callback_action = 1;
	popup_items->item_type       = "<Separator>";
	popup_items->extra_data      = NULL;
	gtk_item_factory_create_item( popup_item, popup_items, NULL, 1);
  }

  /* Extract */
  popup_items->path            = _("/Extract");
  popup_items->accelerator     = NULL;
  popup_items->callback        = callback_popup_menu;
  popup_items->callback_action = 2;
  popup_items->item_type       = "<StockItem>";
  popup_items->extra_data      = GTK_STOCK_REMOVE;
  gtk_item_factory_create_item( popup_item, popup_items, NULL, 1);


  g_free(popup_items);

  return(popup_item);

}






void view_selected_files(gboolean recursive)
{
  GnomeVFSMimeApplication* app=NULL;
  GList*                   l;
  const gchar*             mime;
  gchar**                  argv;
  gchar*                   error_msg, *base;
  gint                     child_pid;

  l = lintar_get_current_selected_files(recursive);
  
  if (l!=NULL) {

	l = g_list_first(l);
	extract_archive( lintar_get_current_filename(), lintar_get_tmp_dir(), 
					 TRUE, FALSE, l);
	lintar_push_statusbar(lintar_get_short_about());
	
	while (l!=NULL) {

	  mime = (const gchar *)gnome_vfs_mime_type_from_name( (const char*)l->data);
	  if (mime!=NULL) {
		app = gnome_vfs_mime_get_default_application( (const char *)mime);
		if (app!=NULL) {

		  // create argv
		  argv = (gchar **)g_malloc( (gulong)(3*sizeof(gchar*)));
		  argv[0] = g_strdup(app->command); 
		  argv[1] = g_strdup_printf("%s/%s", lintar_get_tmp_dir(), (gchar*)l->data);
		  argv[2] = NULL;
		  g_spawn_async( NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, 
						 &child_pid, NULL);
		  g_strfreev(argv);
		  g_free(app);
		} else {
		  /* No default action for file */
		  base = g_path_get_basename((gchar *)l->data);
		  error_msg = g_strdup_printf( _("No default Action for file \"%s\" (%s)"),
									   base, mime);
		  lintar_report_error(error_msg);
		  g_free(error_msg); g_free(base);
		}
		  
		l = g_list_next(l);		
	  }
	}
	
	/* Free list */
	g_list_foreach( l, gchar_list_free, NULL);
	g_list_free(l);

  } else {

	lintar_report_error(_("No files selected"));
  }

}

gboolean copy_file( gchar *from, gchar *to, gboolean overwrite)
{
  gchar *str, **argv;
  gboolean spawn;
  gint i;

  /* Check if file exists already to signal an error */
  if (!overwrite) {
	if (g_file_test(to,G_FILE_TEST_EXISTS)) {
	  str = g_strdup_printf(_("File %s already exists."), to);
	  lintar_report_error(str);
	  g_free(str);
	  return(FALSE);
	}
  }

  /* Copy file 'from' to 'to' (we know it doesn't exist) */
  i=0; argv = (gchar **)g_malloc( (gulong)(10)*sizeof(gchar*) );
  argv[i++] = g_strdup(LINTAR_PROG_CP); 
  argv[i++] = g_strdup("-f");
  argv[i++] = g_strdup(from);
  argv[i++] = g_strdup(to);
  argv[i] = NULL;
  spawn = thread_g_spawn_sync( NULL, argv, NULL, 0, NULL, NULL,  NULL,
							   NULL, NULL, NULL);
  g_strfreev(argv);
  
  return(spawn);
}







