/* ************************************************************************** */

#include "lintar.h"




static void callback_destroy( GtkWidget *widget,
							  gpointer   data )
{
  lintar_destroy_all();
  
  /* Just let's go */
  gtk_main_quit();

}



void callback_notebook_switch_page( GtkNotebook *notebook,
									GtkNotebookPage *page,
									gint page_num,
									gpointer user_data)
{
  lintar_set_current_archive(page_num);						
  
  
  /* In case you want to change the statusbar */
  /*
  guint id, mid;
  gchar *str, *base;

  base = g_path_get_basename( lintar_get_current_filename() ); 
  str = g_strdup_printf( _("Archive %s: %d bytes, %d size"), base, 
						lintar_get_current_tsize(), 
						lintar_get_current_nfiles() );
  id = gtk_statusbar_get_context_id( GTK_STATUSBAR(lintar_data.statusbar), 
									 "Lintar");
  mid = gtk_statusbar_push( GTK_STATUSBAR(lintar_data.statusbar), id, str);

  g_free(str); g_free(base);
  */
}

static void callback_toolbar( GtkWidget *button,
							  gpointer   data)
{
  
  /* Call the action to do */
  lintar_callback_action((gchar*)data);
  
}

static void callback_menu( gpointer  callback_data,
						   guint     callback_action,
						   GtkWidget *widget)
{
  gchar *str;
  
  switch (callback_action) {
  case  0: str = g_strdup("<menu>/File");                break;
  case  1: str = g_strdup("<menu>/File/Open");           break;
  case  2: str = g_strdup("<menu>/File/sep0");           break;
  case  3: str = g_strdup("<menu>/File/View");           break;
  case  4: str = g_strdup("<menu>/File/Extract");        break;
  case  5: str = g_strdup("<menu>/File/sep1");           break;
  case  6: str = g_strdup("<menu>/File/Close");          break;
  case  7: str = g_strdup("<menu>/File/Quit");           break;
  case  8: str = g_strdup("<menu>/Options");             break;
  case  9: str = g_strdup("<menu>/Options/Preferences"); break;
  case 10: str = g_strdup("<menu>/Help");                break;
  case 11: str = g_strdup("<menu>/Help/About");          break;
  default: str = g_strdup("<menu>/callback_action");     break;
  }
  
  /* Call the action to do */
  lintar_callback_action(str);
  
  g_free(str);
}



int main( int argc, char *argv[])
{

  GtkWidget*      window, *table, *label, *statusbar, *notebook, *pbar, *hbox;
  GtkWidget*      toolbar, *toolbar_open, *toolbar_quit, *toolbar_close;
  GtkWidget*      toolbar_extract, *toolbar_view, *dialog;
  GtkAccelGroup  *accel_group;      
  GtkItemFactory *item_factory;
  GtkRequisition  requisition;
  gulong          notebook_callback_id;
  gint            i, init;
  gboolean        extract=FALSE;
  

  static GtkItemFactoryEntry menu_items[] = {
	{NULL,NULL,        0,             0,"<Branch>"                          }, /* "/_File" */ 
	{NULL,"<control>O",callback_menu, 1,"<StockItem>",GTK_STOCK_OPEN        }, /* "/File/_Open" */
	{NULL,NULL,        callback_menu, 2,"<Separator>"                       }, /* "/File/sep0" */
	{NULL,"<control>V",callback_menu, 3,"<StockItem>",GTK_STOCK_EXECUTE     }, /* "/File/_View" */
	{NULL,"<control>E",callback_menu, 4,"<StockItem>",GTK_STOCK_REMOVE      }, /* "/File/_Extract" */
	{NULL,NULL,        callback_menu, 5,"<Separator>"                       }, /* "/File/sep1" */
	{NULL,"<control>C",callback_menu, 6,"<StockItem>",GTK_STOCK_CLOSE       }, /* "/File/_Close" */
	{NULL,"<control>Q",callback_menu, 7,"<StockItem>",GTK_STOCK_QUIT        }, /* "/File/_Quit" */
	{NULL,NULL,        0,             8,"<Branch>"                          }, /* "/_Options" */ 
	{NULL,"<control>P",callback_menu, 9,"<StockItem>",GTK_STOCK_PREFERENCES }, /* "/Options/_Preferences" */
	{NULL,NULL,        0,            10,"<Branch>"                          }, /* "/_Help" */
	{NULL,NULL,        callback_menu,11,"<StockItem>",GTK_STOCK_DIALOG_INFO }, /* "/Help/_About" */
  };

  /* Init gettext support */
  bindtextdomain( "lintar", PACKAGE_LOCALE_DIR);
  textdomain("lintar");


  /* remember to initialize GnomeVFS! */
  if (!gnome_vfs_init()) { fprintf(stderr, _("Could not initialize GnomeVFS\n")); exit(-1); }

  /* Init threads (not needed as gnome_vfs does it already) */
  //g_thread_init(NULL);
  //gdk_threads_init();

  /* Initialise GTK application */
  gtk_init( &argc, &argv);

  /* create window */
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title( GTK_WINDOW(window), "LinTar");
  gtk_signal_connect( GTK_OBJECT(window), "destroy", 
					  GTK_SIGNAL_FUNC(callback_destroy), NULL);
  

  /* Create table for storing all widgets */
  table = gtk_table_new (1, 4, FALSE);
  gtk_container_add( GTK_CONTAINER(window), table);

  /* Create menu items labels using gettext */
  i = 0;
  menu_items[i++].path = _("/_File");
  menu_items[i++].path = _("/File/_Open");
  menu_items[i++].path = _("/File/sep0");
  menu_items[i++].path = _("/File/_View");
  menu_items[i++].path = _("/File/_Extract");
  menu_items[i++].path = _("/File/sep1");
  menu_items[i++].path = _("/File/_Close");
  menu_items[i++].path = _("/File/_Quit");
  menu_items[i++].path = _("/_Options");
  menu_items[i++].path = _("/Options/_Preferences");
  menu_items[i++].path = _("/_Help");
  menu_items[i++].path = _("/Help/_About");


  /* Create the menubar */
  accel_group = gtk_accel_group_new();
  gtk_window_add_accel_group( GTK_WINDOW(window), accel_group);
  g_object_unref(accel_group);
  item_factory = gtk_item_factory_new( GTK_TYPE_MENU_BAR, "<main>", accel_group);

  /* Set up item factory to go away with the window */
  g_object_ref( item_factory);
  gtk_object_sink( GTK_OBJECT (item_factory));
  g_object_set_data_full( G_OBJECT(window), "<main>", item_factory,
						  (GDestroyNotify)g_object_unref);

  /* create menu items */
  gtk_item_factory_create_items( item_factory, G_N_ELEMENTS(menu_items),
								 menu_items, window);
  
  gtk_table_attach( GTK_TABLE (table),
					gtk_item_factory_get_widget(item_factory, "<main>"),
					/* X direction */          /* Y direction */
					0, 1,                      0, 1,
					GTK_EXPAND | GTK_FILL,     0,
					0,                         0);

  /* Create Label */
  label = gtk_label_new(_("Click on Open to open a file"));
	
  /* Attach label to table */
  gtk_table_attach( GTK_TABLE (table),
					label,
					/* X direction */       /* Y direction */
					0, 1,                   2, 3,
					GTK_EXPAND | GTK_FILL,  GTK_EXPAND | GTK_FILL,
					0,                      0);
  
  
  /* Notebook */
  notebook = gtk_notebook_new();
  notebook_callback_id = g_signal_connect( GTK_OBJECT(notebook), "switch-page", 
										   G_CALLBACK(callback_notebook_switch_page), 
										   NULL);  

  gtk_table_attach (GTK_TABLE (table),
					notebook,
					/* X direction */       /* Y direction */
					0, 1,                   2, 3,
					GTK_EXPAND | GTK_FILL,  GTK_EXPAND | GTK_FILL,
					0,                      0);
  
 
  /* Create the toolbar */
  toolbar = gtk_toolbar_new ();
  
  toolbar_open = gtk_toolbar_insert_stock( GTK_TOOLBAR (toolbar), GTK_STOCK_OPEN, 
										   _("Open Archive"), NULL,         /* Tooltip */ 
										   G_CALLBACK(callback_toolbar), /* Callback */ 
										   "<toolbar>/Open",             /* user data */
										   -1);                          /* -1 means "append" */
  gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
  toolbar_view =  gtk_toolbar_insert_item( GTK_TOOLBAR(toolbar), _("View"),
										   _("View selected files with external program"), 
										   NULL,      
										   gtk_image_new_from_stock( GTK_STOCK_EXECUTE, 
																	 GTK_ICON_SIZE_LARGE_TOOLBAR), 
										   G_CALLBACK(callback_toolbar), /* Callback */
										   "<toolbar>/View",          /* user data */
										   -1);                          /* -1 means "append" */
  toolbar_extract =  gtk_toolbar_insert_item( GTK_TOOLBAR(toolbar), _("Extract"),
											  _("Extract current archive"), NULL,      
											  gtk_image_new_from_stock( GTK_STOCK_REMOVE, 
																		GTK_ICON_SIZE_LARGE_TOOLBAR), 
											  G_CALLBACK(callback_toolbar), /* Callback */
											  "<toolbar>/Extract",          /* user data */
											  -1);                          /* -1 means "append" */
  gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
  toolbar_close = gtk_toolbar_insert_stock( GTK_TOOLBAR (toolbar), GTK_STOCK_CLOSE, 
											_("Close current archive"),
											NULL, G_CALLBACK (callback_toolbar),
											"<toolbar>/Close",        /* user data */
											-1);                      /* -1 means "append" */
  toolbar_quit = gtk_toolbar_insert_stock( GTK_TOOLBAR (toolbar), GTK_STOCK_QUIT, 
										   _("Quit LinTar"),
										   NULL, G_CALLBACK (callback_toolbar),
										   "<toolbar>/Quit",          /* user data */
										   -1);                       /* -1 means "append" */
  gtk_table_attach( GTK_TABLE (table),
					toolbar,
					/* X direction */       /* Y direction */
					0, 1,                   1, 2,
					GTK_EXPAND | GTK_FILL,  0,
					0,                      0);

  /* Create status and progress bar */
  hbox = gtk_hbox_new( FALSE, 5);
  statusbar = gtk_statusbar_new ();
  pbar = gtk_progress_bar_new();

  gtk_statusbar_set_has_resize_grip( GTK_STATUSBAR(statusbar), FALSE);

  /* Pack status and progress bar */
  /* FIXME: There must be a better way to do this, I'm sure */
  gtk_box_pack_start( GTK_BOX(hbox), statusbar, TRUE, TRUE, 0);
  gtk_widget_size_request( statusbar, &requisition);
  gtk_widget_set_size_request( pbar, -1, requisition.width);
  gtk_box_pack_end( GTK_BOX(hbox), pbar, FALSE, FALSE, 0);
  
  gtk_table_attach( GTK_TABLE(table),
					hbox,
					/* X direction */       /* Y direction */
					0, 1,                   3, 4,
					GTK_EXPAND | GTK_FILL,  0,
					0,                      0);
  
  
  /* Initialise global data */
  lintar_data_init( window, label, notebook, notebook_callback_id, 
					statusbar, pbar, toolbar, toolbar_open, toolbar_view, 
					toolbar_extract, toolbar_close, toolbar_quit, 
					item_factory);

  /* Initialize window attributes */
  gtk_window_set_default_size (GTK_WINDOW (window), 650, 400);
  
  /* Set some widgets insensitive */
  lintar_set_widgets_sensitive(FALSE);

  /* Initialise progress bar */
  lintar_set_progressbar(0.0);

  /* Set default window size and position */
  if (lintar_data.width>0) {
	gtk_window_set_default_size( GTK_WINDOW(window), lintar_data.width, lintar_data.height);
	/* Check default move is not bigger than the screen */
	if ( (lintar_data.root_x<gdk_screen_width()) && (lintar_data.root_y<gdk_screen_height()) ) {
	  gtk_window_move( GTK_WINDOW(window), lintar_data.root_x, lintar_data.root_y);
	}
  }

  /* Show everything */
  gtk_widget_show_all(window);

  /* Open input args */
  if (argc>1) {

	init=1;

	/* Open extract dialog after opening files */
	if (g_ascii_strcasecmp(argv[1],"--extract")==0) {
	  extract=TRUE;
	  init++;
	}

	/* Check for a really big number of input files */
	if ((argc-1)>lintar_get_max_initial_files()) {
	  
	  /* Show dialog with information */
	  dialog = gtk_message_dialog_new( GTK_WINDOW(lintar_get_main_window()),
									   GTK_DIALOG_DESTROY_WITH_PARENT,
									   GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
									   _("About to open %d archives, Open anyway?"),
									   (argc-init) );
	  switch(gtk_dialog_run(GTK_DIALOG(dialog))) {
      case GTK_RESPONSE_YES:
		for (i=init; i<argc; i++) {
		  open_archive( argv[i] );
		}
		break;
      default:
         break;
	  }
	  gtk_widget_destroy (dialog);
	  
	} else {
	  
	  for (i=init; i<argc; i++) {
		open_archive( argv[i] );
	  }

	}

	if (extract==TRUE) {
	  /* Open extract dialog after opening some files */
	  if (lintar_get_current_filename()!=NULL) {
		dialog = create_extract_dialog( lintar_get_main_window(), 
										lintar_get_current_filetype());
		
		/* Display that dialog */
		gtk_widget_show_all(dialog);
	  }
	}

  }

  /* Main loop */ 
  gtk_main();

  /* Shutdown GnomeVFS */
  gnome_vfs_shutdown();

  /* Just let's go */
  return(0);

}








