/* ************************************************************************** */

#include "lintar_global.h"

/* Declare global lintar_data */
ST_LinTar_Data lintar_data;

void lintar_data_init( GtkWidget *window, GtkWidget *label, GtkWidget *notebook,
					   gulong notebook_callback_id, 
					   GtkWidget *statusbar, GtkWidget *pbar, GtkWidget *toolbar, 
					   GtkWidget *toolbar_open, GtkWidget *toolbar_view, 
					   GtkWidget *toolbar_extract,
					   GtkWidget *toolbar_close, GtkWidget *toolbar_quit, 
					   GtkItemFactory *item_factory)
{
  gchar pc[LINTAR_MAX_LENGTH], *str;
  FILE *fp;
  static gchar tmp_dir[] = "/tmp/LinTar.XXXXXX";
  
  /* GTK Widgets */
  lintar_data.window = window;
  lintar_data.label = label; 

  lintar_data.notebook = notebook;
  lintar_data.notebook_callback_id = notebook_callback_id;

  lintar_data.toolbar = toolbar;
  lintar_data.toolbar_open = toolbar_open;
  lintar_data.toolbar_view = toolbar_view;
  lintar_data.toolbar_extract = toolbar_extract;
  lintar_data.toolbar_close = toolbar_close;
  lintar_data.toolbar_quit = toolbar_quit;
  lintar_data.item_factory = item_factory;

  /* Satus and Progress bar */
  lintar_data.statusbar = statusbar;
  lintar_data.statusbar_id = (gint) gtk_statusbar_get_context_id( GTK_STATUSBAR( lintar_data.statusbar), "lintar");
  lintar_data.progressbar = pbar;
  lintar_data.statusbar_total = 0;

  /* Archive Data */
  lintar_data.current_archive = -1;
  lintar_data.list = NULL;

  if (!g_thread_supported()) {
	fprintf( stderr, _("Glib thread system not supported!"));
  }
  lintar_data.mutex = g_mutex_new();
  lintar_data.thread = FALSE;

  lintar_data.short_about = g_strconcat( "LinTar v.", VERSION, _(" by Javier Ruiz Hidalgo"), NULL);
  lintar_data.long_about  = g_strdup("LinTar v." VERSION "\n\nJavier Ruiz Hidalgo <jrh@gps.tsc.upc.es>");

  lintar_data.tmp_dir = mkdtemp(tmp_dir);

  lintar_data.pixmaps_lst = (gchar **)g_malloc((gulong)(10*sizeof(gchar*)));
  lintar_data.pixmaps_lst[0] = g_strdup("/usr/share/pixmaps/document-icons/gnome-%s.png");
  lintar_data.pixmaps_lst[1] = g_strdup("/usr/share/pixmaps/nautilus/mytheme/gnome-%s.png");
  lintar_data.pixmaps_lst[2] = g_strdup("/usr/share/pixmaps/nautilus/gnome-%s.png");
  lintar_data.pixmaps_lst[3] = g_strdup("/usr/share/pixmaps/gnome-%s.png");
  lintar_data.pixmaps_lst[4] = g_strdup("/usr/share/pixmaps/%s.png");
  lintar_data.pixmaps_lst[5] = NULL;

  /* At the beggining use defaults */
  lintar_data.treeview = TRUE;
  lintar_data.extract_overwrite = TRUE;
  lintar_data.extract_skip_new = FALSE;
  lintar_data.extract_recursive = TRUE;
  lintar_data.max_initial_files = 10;
  lintar_data.show_dialog_on_error = TRUE;

  lintar_data.prog_tar      = g_strdup(LINTAR_PROG_TAR);
  lintar_data.prog_unzip    = g_strdup(LINTAR_PROG_UNZIP);
  lintar_data.prog_compress = g_strdup(LINTAR_PROG_COMPRESS);
  lintar_data.prog_gzip     = g_strdup(LINTAR_PROG_GZIP);
  lintar_data.prog_bzip2    = g_strdup(LINTAR_PROG_BZIP2);

  lintar_data.width  = -1;
  lintar_data.height = -1;
  lintar_data.root_x = -1;
  lintar_data.root_y = -1;

  /* Check for config file */
  str = g_strdup_printf("%s/.lintarrc", g_get_home_dir());
  if (g_file_test(str, G_FILE_TEST_EXISTS)) {
	
	// open the file 
	fp = fopen( str, "r");
	g_free(str);
	if (fp==NULL) {
	  /* Cannot open config file, message user and use defaults */
	  lintar_report_error(_("Couldn't open config file. Using defaults."));

	} else {

	  /* check config version number */
	  fgets(pc, 2040, fp);
	  if ( strcmp( pc, "LinTar config file (version " VERSION ")\n")!=0) {
		str = g_strconcat( _("Config file is not version "), VERSION, _(". Using defaults"), NULL);
		lintar_report_error(str);
		g_free(str);
	  }	else {
	
		/* TREEVIEW */
		fscanf( fp, "%d", &lintar_data.treeview); fgets(pc, LINTAR_MAX_LENGTH-5, fp);
		/* OVERWRITE */
		fscanf( fp, "%d", &lintar_data.extract_overwrite); fgets(pc, LINTAR_MAX_LENGTH-5, fp);
		/* SKIP_NEW */
		fscanf( fp, "%d", &lintar_data.extract_skip_new); fgets(pc, LINTAR_MAX_LENGTH-5, fp);
		/* RECURSIVE */
 		fscanf( fp, "%d", &lintar_data.extract_recursive); fgets(pc, LINTAR_MAX_LENGTH-5, fp);
		/* MAX INITIAL FILES */
 		fscanf( fp, "%d", &lintar_data.max_initial_files); fgets(pc, LINTAR_MAX_LENGTH-5, fp);
		/* SHOW DIALOG ON ERROR */
 		fscanf( fp, "%d", &lintar_data.show_dialog_on_error); fgets(pc, LINTAR_MAX_LENGTH-5, fp);		

		/* PROGRAMS */
 		fscanf( fp, "%1000s", pc); if (strcmp("none",pc)==0) {pc[0]='\0';}
		lintar_set_prog_tar(pc); fgets(pc, LINTAR_MAX_LENGTH-5, fp);		
 		fscanf( fp, "%1000s", pc); if (strcmp("none",pc)==0) {pc[0]='\0';}
		lintar_set_prog_unzip(pc); fgets(pc, LINTAR_MAX_LENGTH-5, fp);		
 		fscanf( fp, "%1000s", pc); if (strcmp("none",pc)==0) {pc[0]='\0';}
		lintar_set_prog_compress(pc); fgets(pc, LINTAR_MAX_LENGTH-5, fp);		
 		fscanf( fp, "%1000s", pc); if (strcmp("none",pc)==0) {pc[0]='\0';}
		lintar_set_prog_gzip(pc); fgets(pc, LINTAR_MAX_LENGTH-5, fp);		
 		fscanf( fp, "%1000s", pc); if (strcmp("none",pc)==0) {pc[0]='\0';}
		lintar_set_prog_bzip2(pc); fgets(pc, LINTAR_MAX_LENGTH-5, fp);		
		
		/* Read window size and position */
		fscanf( fp, "%d", &lintar_data.width); fgets(pc, LINTAR_MAX_LENGTH-5, fp);
		fscanf( fp, "%d", &lintar_data.height); fgets(pc, LINTAR_MAX_LENGTH-5, fp);
		fscanf( fp, "%d", &lintar_data.root_x); fgets(pc, LINTAR_MAX_LENGTH-5, fp);
		fscanf( fp, "%d", &lintar_data.root_y); fgets(pc, LINTAR_MAX_LENGTH-5, fp);

	  }

	  fclose(fp);
	}

  } else {

	/* File doesn't exists, maybe first time LinTar is executed */
	g_free(str);
  }

}


ST_LinTar_Archive *archive_new( gchar *file, guint8 type, gint nfiles, gint tsize, 
								GtkWidget *treeview)
{
  ST_LinTar_Archive *archive;

  archive = (ST_LinTar_Archive *)g_malloc( (gulong)sizeof(ST_LinTar_Archive));

  archive->filename = g_strdup(file);
  archive->type = type;
  archive->nfiles = nfiles;
  archive->tsize = tsize;
  archive->treeview = treeview;

  return(archive);
}

void gchar_list_free(gpointer data, gpointer user_data)
{
  gchar *p;

  p = (gchar *)data;
  g_free(p);

}

void archive_list_free(gpointer data, gpointer user_data)
{
  ST_LinTar_Archive *p;

  p = (ST_LinTar_Archive *)data;
  g_free(p->filename);

}


void lintar_data_save(void)
{
  FILE *fp;
  gchar *str;
  gint root_x, root_y, width, height;

  str = g_strdup_printf("%s/.lintarrc", g_get_home_dir());

  /* Open config file */
  fp = fopen( str, "w");
  g_free(str);

  if (fp==NULL) {
	/* Cannot open config file, message user and use defaults */
	lintar_report_error(_("Couldn't open config file. Not saving preferences."));
	
  } else {

	/* Save config version number */
	fprintf( fp, "LinTar config file (version %s)\n", VERSION);
	
	/* TREEVIEW */
	fprintf( fp, "%d\t\t# Tree or list view\n", lintar_get_treeview());
	/* OVERWRITE */
	fprintf( fp, "%d\t\t# Overwrite files\n", lintar_get_extract_overwrite());
	/* SKIP_NEW */
	fprintf( fp, "%d\t\t# Skip newer files\n", lintar_get_extract_skip_new());
	/* RECURSIVE */
	fprintf( fp, "%d\t\t# Recursive subfolders\n", lintar_get_extract_recursive());
	/* MAX INITIAL FILES */
	fprintf( fp, "%d\t\t# Max initial files from command line\n", lintar_get_max_initial_files());
	/* SHOW DIALOG ON ERROR */
	fprintf( fp, "%d\t\t# Show dialog on errors\n", lintar_get_show_dialog_on_error());
	/* PROGRAMS */
	if ( lintar_get_prog_tar()[0]=='\0' ) {
	  fprintf( fp, "none\t\t# TAR program location\n");
	} else {
	  fprintf( fp, "%s\t\t# TAR program location\n", lintar_get_prog_tar());
	}
	if ( lintar_get_prog_unzip()[0]=='\0' ) {
	  fprintf( fp, "none\t\t# UNZIP program location\n");
	} else {
	  fprintf( fp, "%s\t\t# UNZIP program location\n", lintar_get_prog_unzip());
	}
	if ( lintar_get_prog_compress()[0]=='\0' ) {
	  fprintf( fp, "none\t\t# COMPRESS program location\n");
	} else {
	  fprintf( fp, "%s\t\t# COMPRESS program location\n", lintar_get_prog_compress());
	}
	if ( lintar_get_prog_gzip()[0]=='\0' ) {
	  fprintf( fp, "none\t\t# GZIP program location\n");
	} else {
	  fprintf( fp, "%s\t\t# GZIP program location\n", lintar_get_prog_gzip());
	}
	if ( lintar_get_prog_bzip2()[0]=='\0' ) {
	  fprintf( fp, "none\t\t# BZIP2 program location\n");
	} else {
	  fprintf( fp, "%s\t\t# BZIP2 program location\n", lintar_get_prog_bzip2());
	}

	/* Window positions */
	gtk_window_get_size( GTK_WINDOW(lintar_get_main_window()), &width, &height);
	fprintf(fp, "%d\t\t# Width  main window\n", width);
	fprintf(fp, "%d\t\t# Height main window\n", height);
	gtk_window_get_position( GTK_WINDOW(lintar_get_main_window()), &root_x, &root_y);
	fprintf(fp, "%d\t\t# X position\n", root_x);
	fprintf(fp, "%d\t\t# Y position\n", root_y);

	fclose(fp);
  }

}


void lintar_data_free(void)
{
  gchar **argv;
  gint child_pid;
  
  /* Free mutex */
  g_mutex_free( lintar_data.mutex );

  /* Free about messages */
  g_free(lintar_data.short_about);
  g_free(lintar_data.long_about);

  /* Free progs */
  g_free(lintar_data.prog_tar);
  g_free(lintar_data.prog_unzip);
  g_free(lintar_data.prog_compress);
  g_free(lintar_data.prog_gzip);
  g_free(lintar_data.prog_bzip2);

  /* Free temporal dir */
  argv = (gchar **)g_malloc( (gulong)(4*sizeof(gchar*)));
  argv[0] = g_strdup(LINTAR_PROG_RM); 
  argv[1] = g_strdup("-rf");
  argv[2] = g_strdup(lintar_get_tmp_dir());
  argv[3] = NULL;
  g_spawn_async( NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, &child_pid, NULL);
  g_strfreev(argv);
  
  /* Free pixmap list */
  g_strfreev(lintar_data.pixmaps_lst);

  /* Free list */
  g_list_foreach( lintar_data.list, archive_list_free, NULL);
  g_list_free(lintar_data.list);

}

void lintar_mutex_lock(void)
{
  g_mutex_lock( lintar_data.mutex );
}
void lintar_mutex_unlock(void)
{
  g_mutex_unlock( lintar_data.mutex );
}
void lintar_set_thread(gboolean b)
{
  lintar_mutex_lock();
  lintar_data.thread = b;
  lintar_mutex_unlock();
}
gboolean lintar_get_thread(void)
{
  gboolean b;

  lintar_mutex_lock();
  b = lintar_data.thread;
  lintar_mutex_unlock();
  return(b);
}

gchar *lintar_get_tmp_dir(void) { return(lintar_data.tmp_dir); }
gchar *lintar_get_short_about(void) { return(lintar_data.short_about); }
gchar *lintar_get_long_about(void)  { return(lintar_data.long_about); }

void lintar_set_extract_overwrite(gboolean b)
{
  lintar_data.extract_overwrite = b;
}
gboolean lintar_get_extract_overwrite(void)
{
  return(lintar_data.extract_overwrite);
}

void lintar_set_show_dialog_on_error(gboolean b)
{
  lintar_data.show_dialog_on_error = b;
}
gboolean lintar_get_show_dialog_on_error(void)
{
  return(lintar_data.show_dialog_on_error);
}

void lintar_set_extract_skip_new(gboolean b)
{
  lintar_data.extract_skip_new = b;
}
gboolean lintar_get_extract_skip_new(void)
{
  return(lintar_data.extract_skip_new);
}

void lintar_set_extract_recursive(gboolean b)
{
  lintar_data.extract_recursive = b;
}
gboolean lintar_get_extract_recursive(void)
{
  return(lintar_data.extract_recursive);
}

void lintar_set_max_initial_files(gint i)
{
  lintar_data.max_initial_files = i;
}
gint lintar_get_max_initial_files(void)
{
  return(lintar_data.max_initial_files);
}

GtkWidget *lintar_get_toolbar_view()
{
  return(lintar_data.toolbar_view);
}
GtkWidget *lintar_get_toolbar_extract()
{
  return(lintar_data.toolbar_extract);
}
GtkWidget *lintar_get_toolbar_close()
{
  return(lintar_data.toolbar_close);
}
GtkWidget *lintar_get_label()
{
  return(lintar_data.label);
}
GtkWidget *lintar_get_notebook()
{
  return(lintar_data.notebook);
}
GtkWidget *lintar_get_progressbar()
{
  return(lintar_data.progressbar);
}
GtkItemFactory *lintar_get_item_factory()
{
  return(lintar_data.item_factory);
}

void lintar_set_current_archive(gint i)
{
  lintar_data.current_archive = i;
}
gint lintar_get_current_archive(void)
{
  return(lintar_data.current_archive);
}


gchar *lintar_get_current_filename(void)
{
  ST_LinTar_Archive *archive;  

  if (lintar_data.current_archive==-1) {
	return(NULL);
  } else {
	archive = (ST_LinTar_Archive *) g_list_nth_data( lintar_data.list, 
													 lintar_data.current_archive);
	if (archive==NULL) {
	  return(NULL);
	} else {
	  return(archive->filename);
	}
  }
}

guint8 lintar_get_current_filetype(void)
{
  ST_LinTar_Archive *archive;  
  
  if (lintar_data.current_archive==-1) {
	return(-1);
  } else {
	archive = (ST_LinTar_Archive *) g_list_nth_data( lintar_data.list, 
													 lintar_data.current_archive);
	if (archive==NULL) {
	  return(-1);
	} else {
	  return(archive->type);
	}
  }

  return(-1);
}

gint lintar_get_current_nfiles(void)
{
  ST_LinTar_Archive *archive;  
  
  if (lintar_data.current_archive==-1) {
	return(-1);
  } else {
	archive = (ST_LinTar_Archive *) g_list_nth_data( lintar_data.list, 
													 lintar_data.current_archive);
	if (archive==NULL) {
	  return(-1);
	} else {
	  return(archive->nfiles);
	}
  }
}

gint lintar_get_current_tsize(void)
{
  ST_LinTar_Archive *archive;  

  if (lintar_data.current_archive==-1) {
	return(-1);
  } else {
	archive = (ST_LinTar_Archive *) g_list_nth_data( lintar_data.list, 
													 lintar_data.current_archive);
	if (archive==NULL) {
	  return(-1);
	} else {
	  return(archive->tsize);
	}

  }
}

GtkWidget *lintar_get_current_treeview(void)
{
  ST_LinTar_Archive *archive;  
  
  if (lintar_data.current_archive==-1) {
	return(NULL);
  } else {
	archive = (ST_LinTar_Archive *) g_list_nth_data( lintar_data.list, 
													 lintar_data.current_archive);
	if (archive==NULL) {
	  return(NULL);
	} else {
	  return(archive->treeview);
	}

  }
  return(NULL);
}

static void  tree_selection_foreach_n( GtkTreeModel *model,
									   GtkTreePath *path,
									   GtkTreeIter *iter,
									   gpointer data)
{
  gint *n;
  
  n = (gint *)data;
  (*n)++;
}


gint lintar_get_current_n_selected_files(void)
{
  GtkTreeSelection *selection;
  ST_LinTar_Archive *archive;  
  gint n = 0;

  if (lintar_data.current_archive==-1) {
	return(n);
  } else {
	archive = (ST_LinTar_Archive *) g_list_nth_data( lintar_data.list, 
													 lintar_data.current_archive);
	if (archive==NULL) {
	  return(n);
	} else {
	  
	  /* Get selected tree */
	  selection = gtk_tree_view_get_selection( GTK_TREE_VIEW(lintar_get_current_treeview()) );
	  
	  /* Get a string with all files */
	  gtk_tree_selection_selected_foreach( selection, tree_selection_foreach_n, 
										   (gpointer)&n);
	  
	  return(n);
	}

  }
  return(FALSE);
}




static void  tree_selection_foreach_files_recursive( GtkTreeModel *model,
													 GtkTreePath *path,
													 GtkTreeIter *iter,
													 gpointer data)
{
  ST_LinTar_Files_Recursive *r;
  GList *l;
  gchar *name = NULL, *str, *string_path;
  GtkTreePath *child_path;
  GtkTreeIter child_iter;
  gint Nchildren, i;
													   
  r = (ST_LinTar_Files_Recursive *)data;

  if (gtk_tree_model_iter_has_child(model,iter)) {
	
	/* Has children, Do not include this node but its children */
	Nchildren = gtk_tree_model_iter_n_children(model,iter);
	
	string_path = gtk_tree_path_to_string(path);
	
	for(i=0;i<Nchildren;i++) {

	  /* Get path for current child */
	  str = g_strdup_printf("%s:%d", string_path, i);
	  
	  /* Get child */
	  if (gtk_tree_model_get_iter_from_string( model, &child_iter, str)==TRUE) {
		
		child_path = gtk_tree_model_get_path( model, &child_iter);
		
		// Call function recursively
		tree_selection_foreach_files_recursive(model,child_path,&child_iter,data);

		// Free allocated data
		gtk_tree_path_free(child_path);
		
	  }
	  
	  g_free(str);
	  
	}

	g_free(string_path);
	
  } else {
	
	gtk_tree_model_get( model, iter, FULLNAME_COLUMN, &name, -1);
	if (name!=NULL) {	  
	  /* include name on list only if it hasn't been included before */
	  l = g_list_find( r->list, iter->user_data);	  
	  if (l==NULL) {
		r->namelst = g_list_append( r->namelst, name);
		r->list = g_list_append( r->list, iter->user_data);
	  }

	}

  }

}

static void  tree_selection_foreach_files( GtkTreeModel *model,
										   GtkTreePath *path,
										   GtkTreeIter *iter,
										   gpointer data)
{
  gchar *name;
  GList **pplist;

  /* Init variables */
  pplist = (GList **)data;
  name = NULL;
  
  if (gtk_tree_model_iter_has_child(model,iter)) {
	
  } else {
	
	gtk_tree_model_get( model, iter, FULLNAME_COLUMN, &name, -1);
	if (name!=NULL) {
	  (*pplist) = g_list_append( (*pplist), name);
	}
	
  }
  
}


GList *lintar_get_current_selected_files(gboolean recursive)
{
  GtkTreeSelection *selection;
  ST_LinTar_Archive *archive;  
  GList *list;
  ST_LinTar_Files_Recursive r;
	
  list = NULL;

  if (lintar_data.current_archive==-1) {
	return(list);
  } else {
	archive = (ST_LinTar_Archive *) g_list_nth_data( lintar_data.list, 
													 lintar_data.current_archive);
	if (archive==NULL) {
	  return(list);
	} else {
	  
	  /* Get selected tree */
	  selection = gtk_tree_view_get_selection( GTK_TREE_VIEW(lintar_get_current_treeview()) );
	  
	  /* Get a list with all string_paths selected */
	  if (recursive) {
		r.namelst = NULL; r.list = NULL;
		gtk_tree_selection_selected_foreach( selection, 
											 tree_selection_foreach_files_recursive, 
											 (gpointer)(&r));
		g_list_free(r.list);
		return(r.namelst);
	  } else {
		gtk_tree_selection_selected_foreach( selection, 
											 tree_selection_foreach_files,
											 (gpointer)(&list));
		return(list);
	  }
	  
	}
	
  }
  
  return(list);
}





void lintar_add_archive( gchar *filename, guint8 type, gint nfiles, gint tsize, 
						 GtkWidget *treeview, GtkWidget *parent)
{
  ST_LinTar_Archive *archive;


  /* Create archive */
  archive = archive_new( filename, type, nfiles, tsize, treeview);

  lintar_data.list = g_list_append( lintar_data.list, (gpointer)archive);
  
  lintar_data.current_archive = g_list_length(lintar_data.list)-1;

  /* Attach scroll+treeview to notebook */
  lintar_notebook_add_page(parent, filename);

}


/*
void lintar_close_current_archive(void)
{
  lintar_close_archive( gtk_notebook_current_page(GTK_NOTEBOOK(lintar_get_notebook())) );
}
*/

void lintar_close_archive(gint page_num)
{
  GList *list;

  /* Delete current archive in list */
  list = g_list_nth( lintar_data.list, page_num);

  if (list!=NULL) {
	
	/* Remove archive list */
	archive_list_free(list->data, NULL);
	lintar_data.list = g_list_delete_link( lintar_data.list, list);
	
	/* Change current archive if needed */
	if ( gtk_notebook_current_page(GTK_NOTEBOOK(lintar_get_notebook())) >= page_num ) {
	  lintar_data.current_archive--;
	}
 
	/* Delete notebook page */
	gtk_signal_disconnect( GTK_NOTEBOOK(lintar_get_notebook()), 
						   lintar_data.notebook_callback_id);
	gtk_notebook_remove_page( GTK_NOTEBOOK(lintar_get_notebook()), page_num);
	lintar_data.notebook_callback_id = g_signal_connect( GTK_NOTEBOOK(lintar_get_notebook()), 
														   "switch-page", 
														   G_CALLBACK(callback_notebook_switch_page), 
														   NULL);  

	/* Shadow buttons if needed */
	if (g_list_length(lintar_data.list)==0) {
	  lintar_data.current_archive = -1;
	  lintar_set_widgets_sensitive(FALSE);
	}

  }

}

GtkWidget *lintar_get_main_window(void)
{
  return(lintar_data.window);
}
gboolean lintar_get_treeview(void)
{
  return(lintar_data.treeview);
}
void lintar_set_treeview(gboolean b)
{
  lintar_data.treeview = b;
}


void lintar_pop_statusbar(void)
{
  gtk_statusbar_pop( GTK_STATUSBAR(lintar_data.statusbar), 
					 lintar_data.statusbar_id);
  lintar_data.statusbar_total--;
}

void lintar_push_statusbar(gchar *message)
{
  /* Remove all previous messages */
  while (lintar_data.statusbar_total>0) {
  	lintar_pop_statusbar();
  }
  
  /* Add new message */
  gtk_statusbar_push( GTK_STATUSBAR(lintar_data.statusbar),
					  lintar_data.statusbar_id, message);

  lintar_data.statusbar_total++;

  while (gtk_events_pending()) gtk_main_iteration();

}

void lintar_set_progressbar( gdouble d)
{
  gchar *str;
  gint i;

  i = (gint)(d*100.0);
  
  gtk_progress_bar_set_fraction( GTK_PROGRESS_BAR(lintar_data.progressbar), d);

  if (i==0) {
	gtk_progress_bar_set_text( GTK_PROGRESS_BAR(lintar_data.progressbar), "");
  } else if (i==100) {
	str = g_strdup_printf("100%%");
	gtk_progress_bar_set_text( GTK_PROGRESS_BAR(lintar_data.progressbar), str);
	g_free(str);
  } else {
	str = g_strdup_printf("%02d%%", i);
	gtk_progress_bar_set_text( GTK_PROGRESS_BAR(lintar_data.progressbar), str);
	g_free(str);
  }

  while (gtk_events_pending()) gtk_main_iteration();

}



void lintar_report_error(gchar *error_msg) 
{
  GtkWidget *dialog;
  gchar *error, *pc;


  /* Report the error */
  if (error_msg==NULL) {
	error = g_strdup(_("An error ocurred :-?"));
  } else {
	error = g_strdup(error_msg);
  }
  
  if (lintar_get_show_dialog_on_error()) {
	
	/* Not very large please */
	if ( strlen(error) > 100) { error[99] = '\0'; }
  
	/* Show error on dialog */
	dialog = gtk_message_dialog_new( GTK_WINDOW(lintar_get_main_window()),
									 GTK_DIALOG_DESTROY_WITH_PARENT,
									 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
									 error);
	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);
	lintar_push_statusbar(lintar_get_short_about());
	lintar_set_progressbar( 0.0 );
	
  } else {
	
	/* Do not use \n in error message */
	pc = index( error, '\n');
	if (pc!=NULL) { pc[0] = '\0'; }
	
	/* Update Status bar */
	lintar_push_statusbar(error);
	lintar_set_progressbar(0.0);
	
  }

  g_free(error);
  
}

gchar *lintar_get_prog_tar(void)
{ 
  return(lintar_data.prog_tar);
}
void lintar_set_prog_tar(gchar *p)
{ 
  g_free(lintar_data.prog_tar); 
  lintar_data.prog_tar = g_strdup(p);
}
gchar *lintar_get_prog_unzip(void)
{ 
  return(lintar_data.prog_unzip);
}
void lintar_set_prog_unzip(gchar *p)
{ 
  g_free(lintar_data.prog_unzip); 
  lintar_data.prog_unzip = g_strdup(p);
}
gchar *lintar_get_prog_compress(void)
{ 
  return(lintar_data.prog_compress);
}
void lintar_set_prog_compress(gchar *p)
{ 
  g_free(lintar_data.prog_compress); 
  lintar_data.prog_compress = g_strdup(p);
}
gchar *lintar_get_prog_gzip(void)
{ 
  return(lintar_data.prog_gzip);
}
void lintar_set_prog_gzip(gchar *p)
{ 
  g_free(lintar_data.prog_gzip); 
  lintar_data.prog_gzip = g_strdup(p);
}
gchar *lintar_get_prog_bzip2(void)
{ 
  return(lintar_data.prog_bzip2); 
}
void lintar_set_prog_bzip2(gchar *p)
{ 
  g_free(lintar_data.prog_bzip2); 
  lintar_data.prog_bzip2 = g_strdup(p);
}













