#ifndef _LINTAR_GLOBAL_H_
#define _LINTAR_GLOBAL_H_

// General includes
#include <gtk/gtk.h>
#include <libgnomevfs/gnome-vfs.h>
#include <libgnomevfs/gnome-vfs-mime-handlers.h>

// Local includes
#include <config.h>
#include "lintar.h"
#include "general.h"

/* Gettext support */

/* NO SUPPORT
#define _(String) (String)
#define N_(String) (String)
#define textdomain(Domain)
#define bindtextdomain(Package, Directory)
*/

/* NORMAL SUPPORT */
#include <libintl.h> 
#define _(String) g_locale_to_utf8(gettext(String), -1, NULL, NULL, NULL)
//#define _(String) gettext(String) 
#define gettext_noop(String) (String) 
#define N_(String) gettext_noop(String) 



/* Defines */
#define LINTAR_MAX_LENGTH 10240    /* be safe :-) */

/* Structs */
struct _LinTar_Files_Recursive {
  GList *namelst;
  GList *list;     /* Temporal list */
};
typedef struct _LinTar_Files_Recursive ST_LinTar_Files_Recursive;

struct _LinTar_Archive {
  gchar  *filename;
  guint8 type;
  gint   tsize;
  gint   nfiles;
  GtkWidget *treeview;
};
typedef struct _LinTar_Archive ST_LinTar_Archive;


struct _LinTar_Data {
 
  /* main window */
  GtkWidget *window;

  /* label (behind notebook) */
  GtkWidget *label; 

  /* notebook to add new open files */
  GtkWidget *notebook;
  gulong notebook_callback_id;

  /* Status and progress bar */
  GtkWidget *statusbar;
  gint statusbar_id, statusbar_total;
  GtkWidget *progressbar;

  /* Toolbar widgets */
  GtkWidget *toolbar, *toolbar_open, *toolbar_view, *toolbar_extract;
  GtkWidget *toolbar_quit, *toolbar_close;

  /* Item factory */
  GtkItemFactory *item_factory;

  /* Archive names */
  gint current_archive;    /* Current Archive in list */
  GList *list;

  /* Thread Save */
  gboolean  thread;
  GMutex   *mutex;

  gint width, height, root_x, root_y;

  /* Locations for programs */
  gchar *prog_tar, *prog_unzip, *prog_compress, *prog_gzip, *prog_bzip2;

  /* Global variables */
  gchar  *short_about;           /* Short message about lintar                */
  gchar  *long_about;            /* Long messages about lintar                */
  gchar **pixmaps_lst;           /* List of pixmaps path                      */
  gchar  *tmp_dir;               /* Temporal directory                        */

  /* Settings and Options for LinTar */
  gint     max_initial_files;    /* Maximum Number of archives to open        */
  gboolean treeview;             /* Tree or List View                         */
  gboolean extract_overwrite;    /* Overwrite existing files                  */
  gboolean extract_skip_new;     /* Skip newer files when extracting archives */
  gboolean extract_recursive;    /* Extract selected directories recursively  */
  gboolean show_dialog_on_error; /* Show dialog on errors (or statusbar)      */

};
typedef struct _LinTar_Data ST_LinTar_Data;

/* Enums */
enum {
  NAME_COLUMN = 0,         /* File basename                  */
  PATH_COLUMN,             /* Path (dirname)                 */
  FULLNAME_COLUMN,         /* Fullname (output to tar,zip)   */
  PERMISSIONS_COLUMN,      /* Permissions                    */
  OWN_COLUMN,              /* owner/group                    */
  SIZE_COLUMN,             /* file size (uncompressed)       */
  SIZE_COMPRESSED_COLUMN,  /* Size compressed (%)            */
  DATE_COLUMN,             /* Full Date                      */ 
  LINK_COLUMN,             /* Symbolic link                  */
  ICON_COLUMN,             /* gdkpixbuf associated with file */
  NUM_COLUMNS};            /* Total Number of columns        */

enum {
  LINTAR_TYPE_UNKNOWN,
  LINTAR_TYPE_FOLDER,
  LINTAR_TYPE_LINK,
  LINTAR_TYPE_FILE,
  LINTAR_TYPE_TAR,
  LINTAR_TYPE_TAR_Z,
  LINTAR_TYPE_TAR_GZ,
  LINTAR_TYPE_TAR_BZ2,
  LINTAR_TYPE_ZIP,
  LINTAR_TYPE_GZ,
  LINTAR_TYPE_BZ2,
  LINTAR_TYPE_Z
};


/* Declare a global linzip_data */
extern ST_LinTar_Data lintar_data;

/* Proto definitions */
void lintar_data_init( GtkWidget *window, GtkWidget *label, GtkWidget *notebook, 
					   gulong notebook_callback_id, 
					   GtkWidget *statusbar, GtkWidget *pbar, GtkWidget *toolbar, 
					   GtkWidget *toolbar_open, GtkWidget *toolbar_view, 
					   GtkWidget *toolbar_extract,
					   GtkWidget *toolbar_close, GtkWidget *toolbar_quit, 
					   GtkItemFactory *item_factory);

void lintar_data_save(void);
void lintar_data_free(void);

void     lintar_set_thread(gboolean b);
gboolean lintar_get_thread(void);
void     lintar_mutex_lock(void);
void     lintar_mutex_unlock(void);

gchar *lintar_get_tmp_dir(void);
gchar *lintar_get_short_about(void);
gchar *lintar_get_long_about(void);

void     lintar_set_extract_overwrite(gboolean b);
gboolean lintar_get_extract_overwrite(void);
void     lintar_set_extract_skip_new(gboolean b);
gboolean lintar_get_extract_skip_new(void);
void     lintar_set_extract_recursive(gboolean b);
gboolean lintar_get_extract_recursive(void);
void     lintar_set_current_archive(gint i);
gint     lintar_get_current_archive(void);
void     lintar_set_treeview(gboolean b);
gboolean lintar_get_treeview(void);
void     lintar_set_show_dialog_on_error(gboolean b);
gboolean lintar_get_show_dialog_on_error(void);
void     lintar_set_max_initial_files(gint i);
gint     lintar_get_max_initial_files(void);

gchar     *lintar_get_current_filename(void);
guint8     lintar_get_current_filetype(void);
gint       lintar_get_current_nfiles(void);
gint       lintar_get_current_tsize(void);
GtkWidget *lintar_get_current_treeview(void);
gboolean   lintar_get_current_n_selected_files(void);
GList     *lintar_get_current_selected_files(gboolean recursive);

GtkWidget      *lintar_get_main_window(void);
GtkWidget      *lintar_get_toolbar_view();
GtkWidget      *lintar_get_toolbar_extract(void);
GtkWidget      *lintar_get_toolbar_close(void);
GtkWidget      *lintar_get_label(void);
GtkWidget      *lintar_get_notebook(void);
GtkWidget      *lintar_get_progressbar(void);
GtkItemFactory *lintar_get_item_factory(void);

void gchar_list_free(gpointer data, gpointer user_data);
void archive_list_free(gpointer data, gpointer user_data);


void lintar_add_archive( gchar *filename, guint8 type, gint nfiles, gint tsize, GtkWidget *treeview, GtkWidget *parent);
void lintar_close_archive(gint i);

void lintar_pop_statusbar(void);
void lintar_push_statusbar(gchar *message);

void lintar_set_progressbar(gdouble d);
void lintar_report_error(gchar *error_msg);

gchar *lintar_get_prog_tar(void);
void   lintar_set_prog_tar(gchar *p);
gchar *lintar_get_prog_unzip(void);
void   lintar_set_prog_unzip(gchar *p);
gchar *lintar_get_prog_compress(void);
void   lintar_set_prog_compress(gchar *p);
gchar *lintar_get_prog_gzip(void);
void   lintar_set_prog_gzip(gchar *p);
gchar *lintar_get_prog_bzip2(void);
void   lintar_set_prog_bzip2(gchar *p);


#endif

