/* ************************************************************************** */

#include "prefs_dialog.h"


static void callback_prefs_dialog_destroy( GtkWidget *widget,
										   gpointer   user_data )
{
  /* Just let's go */
  gtk_widget_destroy(widget);  
}


static void callback_prefs_dialog_cancel( GtkButton *button,
											gpointer user_data)
{
  ST_PrefsDialog_cbData *cbData;
  
  /* User_data to extract dialog callback data */
  cbData = (ST_PrefsDialog_cbData *)user_data;
  
  /* Destroy main extract window */
  gtk_widget_destroy( cbData->window );
  
}

static void callback_prefs_dialog_apply( GtkButton *button,
										   gpointer user_data)
{
  ST_PrefsDialog_cbData *cbData;
  
  /* User_data to extract dialog callback data */
  cbData = (ST_PrefsDialog_cbData *)user_data;

  /* Save actual options as default ones */
  lintar_set_treeview(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cbData->tree)));
  lintar_set_extract_overwrite(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cbData->overwrite)));
  lintar_set_extract_skip_new(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cbData->skip)));
  lintar_set_extract_recursive(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cbData->recursive)));
  lintar_set_max_initial_files(gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(cbData->opts_max_spin)));

  /* Save programs */
  lintar_set_prog_tar((gchar*)gtk_entry_get_text(GTK_ENTRY(cbData->prog_text_tar)));
  lintar_set_prog_unzip((gchar*)gtk_entry_get_text(GTK_ENTRY(cbData->prog_text_unzip)));
  lintar_set_prog_compress((gchar*)gtk_entry_get_text(GTK_ENTRY(cbData->prog_text_compress)));
  lintar_set_prog_gzip((gchar*)gtk_entry_get_text(GTK_ENTRY(cbData->prog_text_gzip)));
  lintar_set_prog_bzip2((gchar*)gtk_entry_get_text(GTK_ENTRY(cbData->prog_text_bzip2)));
 
  /* Shadow apply button */
  gtk_widget_set_sensitive( cbData->apply, FALSE);

}

static void callback_prefs_dialog_ok( GtkButton *button,
										gpointer user_data)
{
  ST_PrefsDialog_cbData *cbData;
  
  /* User_data to extract dialog callback data */
  cbData = (ST_PrefsDialog_cbData *)user_data;

  /* Call apply and then go out */
  callback_prefs_dialog_apply( button, user_data);

  gtk_widget_destroy(cbData->window);

}

static void  callback_prefs_dialog_toggled(GtkToggleButton *togglebutton,
										   gpointer user_data)
{
  ST_PrefsDialog_cbData *cbData;
  
  /* User_data to extract dialog callback data */
  cbData = (ST_PrefsDialog_cbData *)user_data;

  gtk_widget_set_sensitive( cbData->apply, TRUE);
}

static void callback_prefs_dialog_progs_browse( GtkButton *button,
												gpointer user_data)
{
  GtkWidget *dialog, *entry;

  /* Get callback data */
  entry = (GtkWidget *)user_data;

  /* Open file selection dialog */
  dialog = gtk_file_selection_new(_("Select Program"));

  /* Display that dialog */
  switch ( gtk_dialog_run(GTK_DIALOG(dialog)) ) {
  case GTK_RESPONSE_OK:
  case GTK_RESPONSE_ACCEPT:
	gtk_entry_set_text( GTK_ENTRY(entry), 
						gtk_file_selection_get_filename(GTK_FILE_SELECTION(dialog)));
	break;
  default:
	break;
  }

  /* Destroy it */
  gtk_widget_destroy (dialog);

}

static void callback_prefs_dialog_progs_spin_changed(GtkSpinButton *spinbutton,
													gpointer user_data)
{
  ST_PrefsDialog_cbData *cbData;
  
  /* User_data to extract dialog callback data */
  cbData = (ST_PrefsDialog_cbData *)user_data;
  
  gtk_widget_set_sensitive( cbData->apply, TRUE);
}

static void callback_prefs_dialog_progs_entry_changed( GtkEditable *editable,
													   gpointer     user_data)
{
  ST_PrefsDialog_cbData *cbData;
  
  /* User_data to extract dialog callback data */
  cbData = (ST_PrefsDialog_cbData *)user_data;
  
  gtk_widget_set_sensitive( cbData->apply, TRUE);
}



GtkWidget *create_prefs_dialog(GtkWidget *parent)
{
  GtkWidget* window, *table, *notebook, *opt_table;
  GtkWidget* opt_tree, *opt_overwrite, *opt_skip, *opt_recursive;
  GtkWidget* opt_max_spin, *opt_max_label;

  GtkWidget* prog_table, *prog_label_tar, *prog_label_unzip, *prog_label_compress;
  GtkWidget* prog_label_gzip, *prog_label_bzip2;
  GtkWidget* prog_text_tar, *prog_text_unzip, *prog_text_compress;
  GtkWidget* prog_text_gzip, *prog_text_bzip2;
  GtkWidget* prog_browse_tar, *prog_browse_unzip, *prog_browse_compress;
  GtkWidget* prog_browse_gzip, *prog_browse_bzip2;

  GtkWidget *button_ok, *button_apply, *button_cancel, *button_box;
  static ST_PrefsDialog_cbData cbData;

  /* 
   * Create Main window
   */
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title( GTK_WINDOW (window), _("Preferences"));
  gtk_signal_connect( GTK_OBJECT (window), "destroy",  
					  GTK_SIGNAL_FUNC(callback_prefs_dialog_destroy), NULL);
  gtk_container_set_border_width( GTK_CONTAINER(window), 5);
  
  /* Create table for storing all widgets */
  table = gtk_table_new(2, 1, FALSE);
  gtk_container_add( GTK_CONTAINER(window), table);
  
  
  /* 
   * Create notebook 
   */
  notebook = gtk_notebook_new();
  gtk_table_attach( GTK_TABLE(table), notebook, 
					/* X direction */          /* Y direction */
					0, 1,                      0, 1,
					GTK_EXPAND | GTK_FILL,     GTK_EXPAND | GTK_FILL,
					0,                         5);



  /* 
   * Options NOTEBOOK
   */
  opt_table = gtk_table_new(5, 2, FALSE);
  gtk_container_set_border_width( GTK_CONTAINER(opt_table), 10);
  
  opt_tree      = gtk_check_button_new_with_mnemonic(_("_Tree view"));
  opt_overwrite = gtk_check_button_new_with_mnemonic(_("_Overwrite existing files"));
  opt_skip      = gtk_check_button_new_with_mnemonic(_("S_kip new files"));
  opt_recursive = gtk_check_button_new_with_mnemonic(_("_Recursive selected subdirectories"));

  opt_max_spin = gtk_spin_button_new( GTK_ADJUSTMENT(gtk_adjustment_new( (gdouble)lintar_get_max_initial_files(), 0.0, 999.0, 1.0, 1.0, -1.0)), 1.0, 0);
  opt_max_label = gtk_label_new(_("Maximum number of open archives from command line"));


  /* Select default options */
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(opt_tree), lintar_get_treeview());
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(opt_overwrite), lintar_get_extract_overwrite());
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(opt_skip), lintar_get_extract_skip_new());
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(opt_recursive), lintar_get_extract_recursive());
  
  gtk_signal_connect( GTK_OBJECT(opt_tree),      "toggled", 
					  GTK_SIGNAL_FUNC(callback_prefs_dialog_toggled), (gpointer)(&cbData));
  gtk_signal_connect( GTK_OBJECT(opt_overwrite), "toggled", 
					  GTK_SIGNAL_FUNC(callback_prefs_dialog_toggled), (gpointer)(&cbData));
  gtk_signal_connect( GTK_OBJECT(opt_skip),      "toggled", 
					  GTK_SIGNAL_FUNC(callback_prefs_dialog_toggled), (gpointer)(&cbData));
  gtk_signal_connect( GTK_OBJECT(opt_recursive), "toggled", 
					  GTK_SIGNAL_FUNC(callback_prefs_dialog_toggled), (gpointer)(&cbData));
  g_signal_connect( GTK_OBJECT(opt_max_spin), "value-changed", 
					G_CALLBACK(callback_prefs_dialog_progs_spin_changed), 
					(gpointer)(&cbData));


  gtk_table_attach( GTK_TABLE(opt_table), opt_tree, 
					0, 2,                      0, 1,
					GTK_EXPAND | GTK_FILL,     GTK_EXPAND | GTK_FILL,
					0,                         0);
  gtk_table_attach( GTK_TABLE(opt_table), opt_overwrite, 
					0, 2,                      1, 2,
					GTK_EXPAND | GTK_FILL,     GTK_EXPAND | GTK_FILL,
					0,                         0);
  gtk_table_attach( GTK_TABLE(opt_table), opt_skip, 
					/* X direction */          /* Y direction */
					0, 2,                      2, 3,
					GTK_EXPAND | GTK_FILL,     GTK_EXPAND | GTK_FILL,
					0,                         0);
  gtk_table_attach( GTK_TABLE(opt_table), opt_recursive, 
					/* X direction */          /* Y direction */
					0, 2,                      3, 4,
					GTK_EXPAND | GTK_FILL,     GTK_EXPAND | GTK_FILL,
					0,                         0);
  gtk_table_attach( GTK_TABLE(opt_table), opt_max_spin , 
					/* X direction */          /* Y direction */
					0, 1,                      4, 5,
					GTK_EXPAND | GTK_FILL,     GTK_EXPAND | GTK_FILL,
					0,                         0);
  gtk_table_attach( GTK_TABLE(opt_table), opt_max_label, 
					/* X direction */          /* Y direction */
					1, 2,                      4, 5,
					GTK_EXPAND | GTK_FILL,     GTK_EXPAND | GTK_FILL,
					5,                         0);
  gtk_notebook_append_page( GTK_NOTEBOOK(notebook), opt_table, 
							gtk_label_new(_("Options")));
					  

  /* 
   * Progs NOTEBOOK
   */
  prog_table = gtk_table_new(5, 3, FALSE);
  gtk_container_set_border_width( GTK_CONTAINER(prog_table), 10);


  /* TAR */
  prog_label_tar = gtk_label_new("Tar :");
  gtk_table_attach( GTK_TABLE(prog_table), prog_label_tar, 
					0, 1,                   0, 1,
					0,                      GTK_EXPAND | GTK_FILL,
					5,                      0);
  prog_text_tar = gtk_entry_new();
  gtk_entry_set_text( GTK_ENTRY(prog_text_tar), lintar_get_prog_tar());
  gtk_signal_connect( GTK_OBJECT(prog_text_tar), "changed",
                      GTK_SIGNAL_FUNC(callback_prefs_dialog_progs_entry_changed),
                      (gpointer)&cbData);
  gtk_table_attach( GTK_TABLE(prog_table), prog_text_tar, 
					1, 2,                   0, 1,
					GTK_EXPAND | GTK_FILL,  GTK_EXPAND | GTK_FILL,
					5,                      0);
  prog_browse_tar = gtk_button_new_with_label("...");
  g_signal_connect( GTK_OBJECT(prog_browse_tar), "clicked", 
					G_CALLBACK(callback_prefs_dialog_progs_browse), 
					(gpointer)(prog_text_tar));
  gtk_table_attach( GTK_TABLE(prog_table), prog_browse_tar, 
					2, 3,                   0, 1,
					0,                      0,
					0,                      0);
  
  /* UNZIP */
  prog_label_unzip = gtk_label_new("Unzip :");
  gtk_table_attach( GTK_TABLE(prog_table), prog_label_unzip, 
					0, 1,                   1, 2,
					0,                      GTK_EXPAND | GTK_FILL,
					5,                      0);
  prog_text_unzip = gtk_entry_new();
  gtk_entry_set_text( GTK_ENTRY(prog_text_unzip), lintar_get_prog_unzip());
  gtk_signal_connect( GTK_OBJECT(prog_text_unzip), "changed",
                      GTK_SIGNAL_FUNC(callback_prefs_dialog_progs_entry_changed),
                      (gpointer)&cbData);
  gtk_table_attach( GTK_TABLE(prog_table), prog_text_unzip, 
					1, 2,                   1, 2,
					GTK_EXPAND | GTK_FILL,  GTK_EXPAND | GTK_FILL,
					5,                      0);
  prog_browse_unzip = gtk_button_new_with_label("...");
  g_signal_connect( GTK_OBJECT(prog_browse_unzip), "clicked", 
					G_CALLBACK(callback_prefs_dialog_progs_browse), 
					(gpointer)(prog_text_unzip));
  gtk_table_attach( GTK_TABLE(prog_table), prog_browse_unzip, 
					2, 3,                   1, 2,
					0,                      0,
					0,                      0);
  /* COMPRESS */
  prog_label_compress = gtk_label_new("Compress :");
  gtk_table_attach( GTK_TABLE(prog_table), prog_label_compress, 
					/* X direction */       /* Y direction */
					0, 1,                   2, 3,
					0,                      GTK_EXPAND | GTK_FILL,
					5,                      0);
  prog_text_compress = gtk_entry_new();
  gtk_entry_set_text( GTK_ENTRY(prog_text_compress), lintar_get_prog_compress());
  gtk_signal_connect( GTK_OBJECT(prog_text_compress), "changed",
                      GTK_SIGNAL_FUNC(callback_prefs_dialog_progs_entry_changed),
                      (gpointer)&cbData);
  gtk_table_attach( GTK_TABLE(prog_table), prog_text_compress, 
					1, 2,                   2, 3,
					GTK_EXPAND | GTK_FILL,  GTK_EXPAND | GTK_FILL,
					5,                      0);
  prog_browse_compress = gtk_button_new_with_label("...");
  g_signal_connect( GTK_OBJECT(prog_browse_compress), "clicked", 
					G_CALLBACK(callback_prefs_dialog_progs_browse), 
					(gpointer)(prog_text_compress));
  gtk_table_attach( GTK_TABLE(prog_table), prog_browse_compress, 
					2, 3,                   2, 3,
					0,                      0,
					0,                      0);

  /* GZIP */
  prog_label_gzip = gtk_label_new("Gzip :");
  gtk_table_attach( GTK_TABLE(prog_table), prog_label_gzip, 
					/* X direction */       /* Y direction */
					0, 1,                   3, 4,
					0,                      GTK_EXPAND | GTK_FILL,
					5,                      0);
  prog_text_gzip = gtk_entry_new();
  gtk_entry_set_text( GTK_ENTRY(prog_text_gzip), lintar_get_prog_gzip());
  gtk_signal_connect( GTK_OBJECT(prog_text_gzip), "changed",
                      GTK_SIGNAL_FUNC(callback_prefs_dialog_progs_entry_changed),
                      (gpointer)&cbData);
  gtk_table_attach( GTK_TABLE(prog_table), prog_text_gzip, 
					1, 2,                   3, 4,
					GTK_EXPAND | GTK_FILL,  GTK_EXPAND | GTK_FILL,
					5,                      0);
  prog_browse_gzip = gtk_button_new_with_label("...");
  g_signal_connect( GTK_OBJECT(prog_browse_gzip), "clicked", 
					G_CALLBACK(callback_prefs_dialog_progs_browse), 
					(gpointer)(prog_text_gzip));
  gtk_table_attach( GTK_TABLE(prog_table), prog_browse_gzip, 
					2, 3,                   3, 4,
					0,                      0,
					0,                      0);

  /* BZIP2 */
  prog_label_bzip2 = gtk_label_new("Bzip2 :");
  gtk_table_attach( GTK_TABLE(prog_table), prog_label_bzip2, 
					/* X direction */       /* Y direction */
					0, 1,                   4, 5,
					0,                      GTK_EXPAND | GTK_FILL,
					5,                      0);
  prog_text_bzip2 = gtk_entry_new();
  gtk_entry_set_text( GTK_ENTRY(prog_text_bzip2), lintar_get_prog_bzip2());
  gtk_signal_connect( GTK_OBJECT(prog_text_bzip2), "changed",
                      GTK_SIGNAL_FUNC(callback_prefs_dialog_progs_entry_changed),
                      (gpointer)&cbData);
  gtk_table_attach( GTK_TABLE(prog_table), prog_text_bzip2, 
					1, 2,                   4, 5,
					GTK_EXPAND | GTK_FILL,  GTK_EXPAND | GTK_FILL,
					5,                      0);
  prog_browse_bzip2 = gtk_button_new_with_label("...");
  g_signal_connect( GTK_OBJECT(prog_browse_bzip2), "clicked", 
					G_CALLBACK(callback_prefs_dialog_progs_browse), 
					(gpointer)(prog_text_bzip2));
  gtk_table_attach( GTK_TABLE(prog_table), prog_browse_bzip2, 
					2, 3,                   4, 5,
					0,                      0,
					0,                      0);

  /* Add prog_table to notebook page */
  gtk_notebook_append_page( GTK_NOTEBOOK(notebook), prog_table, 
							gtk_label_new(_("Programs")));
					  



  /*
   * Add Buttons
   */
  button_box     = gtk_hbutton_box_new();
  button_ok      = gtk_button_new_from_stock(GTK_STOCK_OK);
  button_apply   = gtk_button_new_from_stock(GTK_STOCK_APPLY);
  button_cancel  = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
  
  gtk_widget_set_sensitive( button_apply, FALSE);

  g_signal_connect( GTK_OBJECT(button_ok), "clicked", 
 	 				G_CALLBACK(callback_prefs_dialog_ok),
	 	 			(gpointer)(&cbData));
  g_signal_connect( GTK_OBJECT(button_apply), "clicked", 
 	 				G_CALLBACK(callback_prefs_dialog_apply),
	 	 			(gpointer)(&cbData));
  g_signal_connect( GTK_OBJECT(button_cancel), "clicked", 
	 				G_CALLBACK(callback_prefs_dialog_cancel),
		 			(gpointer)(&cbData));

  gtk_container_set_border_width( GTK_CONTAINER(button_box), 5);
  gtk_button_box_set_layout( GTK_BUTTON_BOX(button_box), GTK_BUTTONBOX_END);
  gtk_box_set_spacing( GTK_BOX(button_box), 10);
 
  gtk_container_add( GTK_CONTAINER(button_box), button_cancel);
  gtk_container_add( GTK_CONTAINER(button_box), button_apply);
  gtk_container_add( GTK_CONTAINER(button_box), button_ok);

  gtk_table_attach( GTK_TABLE(table), button_box, 
					0, 1,                      1, 2,
					GTK_EXPAND | GTK_FILL,     0,
					0,                         0);

  /*
   * Fill cbData
   */
  cbData.window    = window;
  cbData.tree      = opt_tree;
  cbData.overwrite = opt_overwrite;
  cbData.skip      = opt_skip;
  cbData.recursive = opt_recursive;
  cbData.opts_max_spin = opt_max_spin;
  cbData.apply = button_apply;
  cbData.prog_text_tar = prog_text_tar;
  cbData.prog_text_unzip = prog_text_unzip;
  cbData.prog_text_compress = prog_text_compress;
  cbData.prog_text_gzip = prog_text_gzip;
  cbData.prog_text_bzip2 = prog_text_bzip2;


  /* Select OK button as default */
  gtk_widget_grab_focus(button_ok);

  /* Set extract window modal */
  gtk_window_set_modal( GTK_WINDOW(window), TRUE);
  gtk_window_set_transient_for( GTK_WINDOW(window), GTK_WINDOW(parent));

  return(window);

}








