/* ************************************************************************** */

#include "prog.h"

gboolean tree_cmp_func( GtkTreeModel *model,
						GtkTreePath *path,
						GtkTreeIter *iter,
						gpointer data)
{
  gchar *pc;
  ST_comp_data *d;
  gboolean b = FALSE;
  gint depth;

  /* cast data into something meaninful */
  d = (ST_comp_data *)data;
  
  /* get value of node */
  gtk_tree_model_get(model, iter, NAME_COLUMN, &pc, -1);
  
  /* check tree depth */
  depth = gtk_tree_store_iter_depth( GTK_TREE_STORE(model), iter);

  /* Check if node has been already created */
  if (d->depth == depth ) {
	if (strcmp(d->name, pc)!=0) {
	  b = FALSE;
	} else {
	  d->iter = (*iter);
	  d->iter_valid=TRUE;
	  b = TRUE;
	}
  } else {
	b = FALSE;
  }
  
  /* free allocated memory and out */
  g_free(pc);
  return(b);
}



static gboolean tree_selection_func_do_nothing( GtkTreeSelection *selection,
												GtkTreeModel *model,
												GtkTreePath *path,
												gboolean path_currently_selected,
												gpointer data)
{
  return(TRUE);
}


static void tree_selection_recursive( GtkTreeSelection *selection,
									  GtkTreeModel     *model,
									  GtkTreeIter      *iter)
{
  GtkTreeIter Child_Iter;
  gint Nchildren, i;
													   

  /* Select nodes recursevely */
  gtk_tree_selection_select_iter( selection, iter);

  if (gtk_tree_model_iter_has_child( model, iter)) {
	
	/* Has children */
	Nchildren = gtk_tree_model_iter_n_children( model, iter);

	gtk_tree_model_iter_children( model, &Child_Iter, iter);
	tree_selection_recursive( selection, model, &Child_Iter);
	
	for(i=1;i<Nchildren;i++) {
	  gtk_tree_model_iter_next( model, &Child_Iter);
	  tree_selection_recursive( selection, model, &Child_Iter);
	}
		
  }
	
}

static gboolean tree_selection_func( GtkTreeSelection *selection,
									 GtkTreeModel *model,
									 GtkTreePath *path,
									 gboolean path_currently_selected,
									 gpointer data)
{
  GtkTreeIter Iter, Child_Iter;
  gint Nchildren, i;
													   

  if (lintar_get_extract_recursive()) {

	/* Disable recursive function */
	gtk_tree_selection_set_select_function( selection, tree_selection_func_do_nothing, NULL, NULL);

	/* Get corresponding Iter */
	gtk_tree_model_get_iter( model, &Iter, path);

	//gtk_tree_selection_unselect_all(selection);

    if (gtk_tree_model_iter_has_child( model, &Iter)) {
	
	  /* Has children, Do not include this node but its children */
	  Nchildren = gtk_tree_model_iter_n_children( model, &Iter);

	  gtk_tree_model_iter_children( model, &Child_Iter, &Iter);
	  tree_selection_recursive( selection, model, &Child_Iter);

	  for(i=1;i<Nchildren;i++) {
		gtk_tree_model_iter_next( model, &Child_Iter);
		tree_selection_recursive( selection, model, &Child_Iter);
	  }

	}

	gtk_tree_selection_set_select_function( selection, tree_selection_func, NULL, NULL);
	return(TRUE);

  } else {  
	return(TRUE);
  }

}



static gboolean popup_menu(GtkWidget *widget,GdkEventButton *event,gpointer data)
{
  gint x, y;
  GtkItemFactory *popup_item;
  
  if (event->button==3) {
	popup_item = lintar_create_popup_menu();
	gdk_window_get_pointer(NULL, &x, &y, NULL);
	gtk_item_factory_popup( popup_item, x, y, event->button, event->time);
	return(TRUE);
  } else {
	return(FALSE);
  }
}


void  row_activated( GtkTreeView       *treeview,
					 GtkTreePath       *arg1,
					 GtkTreeViewColumn *arg2,
					 gpointer           user_data)
{
  view_selected_files(FALSE);
}


void open_archive(gchar *file)
{
  GtkTreeModel *model = NULL;
  GtkWidget *sw, *treeview;
  gchar *error_msg = NULL, *str, *base;
  gint nfiles, tsize;
  guint8 filetype;

  /* Update Status bar */
  base = g_path_get_basename(file); 
  str = g_strdup_printf(_("Opening archive %s ... "), base);
  lintar_push_statusbar(str);
  g_free(str);

  /* Init Open struct */

  /* Check if file exists or it's a directory */
  if (g_file_test( file, G_FILE_TEST_EXISTS)!=TRUE) {
	error_msg = g_strdup_printf( _("Could not find archive: %s"), file);	
  } else if (g_file_test( file, G_FILE_TEST_IS_DIR)==TRUE) {
	error_msg = g_strdup_printf( _("%s is a directory"), file);
  } else {
	
	/* create Tree model depending file name */
	filetype = lintar_get_compress_file_type(file);
	switch (filetype) {
	
	case LINTAR_TYPE_UNKNOWN:
	  error_msg = g_strdup_printf( _("Unknown archive format for %s"), file);
	  break;
	case LINTAR_TYPE_BZ2:
	case LINTAR_TYPE_GZ:
	case LINTAR_TYPE_Z:
	  model = create_model_single( lintar_get_main_window(), file, filetype,
								   lintar_get_treeview(), &nfiles, &tsize, 
								   &error_msg);
	  break;
	case LINTAR_TYPE_TAR:
	  model = create_model_tar( lintar_get_main_window(), file, "",
								lintar_get_treeview(), &nfiles, &tsize, 
								&error_msg);
	  break;
	case LINTAR_TYPE_TAR_Z:
	  model = create_model_tar( lintar_get_main_window(), file, "Z",
								lintar_get_treeview(), &nfiles, &tsize, 
								&error_msg);
	  break;
	case LINTAR_TYPE_TAR_GZ:
	  model = create_model_tar( lintar_get_main_window(), file, "z",
								lintar_get_treeview(), &nfiles, &tsize, 
								&error_msg);
	  break;
	case LINTAR_TYPE_TAR_BZ2:
	  model = create_model_tar( lintar_get_main_window(), file, "j",
								lintar_get_treeview(), &nfiles, &tsize, 
								&error_msg);
	  break;
	case LINTAR_TYPE_ZIP:
	  model = create_model_zip( lintar_get_main_window(), file, 
								lintar_get_treeview(), &nfiles, &tsize, 
								&error_msg);
	  break;
	default:
	  error_msg = g_strdup_printf( _("Unknown archive format for %s"), file);
	  break;
	}

  }
	
  if ( (model==NULL) || (error_msg!=NULL) ) {
	
	/* Report the error */
	lintar_report_error(error_msg);
	g_free(error_msg);
	
  } else {

	/* Scroll widget */
	sw = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
										 GTK_SHADOW_ETCHED_IN);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
									GTK_POLICY_AUTOMATIC,
									GTK_POLICY_AUTOMATIC); 
		
	/* create tree view */
	treeview = gtk_tree_view_new_with_model(model);
	g_object_unref(G_OBJECT(model));
	gtk_tree_view_set_rules_hint( GTK_TREE_VIEW(treeview), TRUE);
	gtk_tree_selection_set_mode( gtk_tree_view_get_selection( GTK_TREE_VIEW(treeview)),
								 GTK_SELECTION_MULTIPLE);
	//gtk_tree_selection_set_select_function( gtk_tree_view_get_selection( GTK_TREE_VIEW(treeview)),
	//										tree_selection_func, NULL, NULL);

	
	switch (lintar_get_compress_file_type(file)) {
	case LINTAR_TYPE_Z:
	case LINTAR_TYPE_GZ:
	case LINTAR_TYPE_BZ2:	  
	  add_columns_single( GTK_TREE_VIEW(treeview), lintar_get_treeview());
	  break;
	case LINTAR_TYPE_TAR:
	case LINTAR_TYPE_TAR_Z:
	case LINTAR_TYPE_TAR_GZ:
	case LINTAR_TYPE_TAR_BZ2:
	  add_columns_tar( GTK_TREE_VIEW(treeview), lintar_get_treeview());
	  break;
	case LINTAR_TYPE_ZIP:
	  add_columns_zip( GTK_TREE_VIEW(treeview), lintar_get_treeview());
	  break;
	case LINTAR_TYPE_UNKNOWN:
	default:
	  fprintf( stderr, _("Shouldn't be here :-)"));
	  exit(-1);
	  break;
	}

	gtk_container_add( GTK_CONTAINER(sw), treeview);
	  
	g_signal_connect(G_OBJECT(treeview), "realize", G_CALLBACK(gtk_tree_view_expand_all), NULL);
	gtk_signal_connect(GTK_OBJECT(treeview), "row-activated", G_CALLBACK(row_activated), NULL);
	gtk_signal_connect(GTK_OBJECT(treeview),"button_press_event",GTK_SIGNAL_FUNC(popup_menu),NULL);
  

	/* Copy filename into lintar data struct */ 
	lintar_add_archive( file, lintar_get_compress_file_type(file), 
						nfiles, tsize, treeview, sw );

	/* Set some toolbar and menu widgets sensitive if needed */
	/* FIXME: Only do this on first file opened! */
	lintar_set_widgets_sensitive(TRUE);
		
	/* Update Status bar */
	str = g_strdup_printf(_("%s: Opened, %d bytes, %d files."), base, tsize, nfiles);
	lintar_push_statusbar(str);
	g_free(str);

	lintar_set_progressbar( 0.0 );
	
	/* Show everything */
	gtk_widget_show_all( lintar_get_main_window() );

	/* Set current page to the opened one */
	lintar_select_notebook_current_page();

  }  

  g_free(base);

}


void extract_archive( gchar *file, gchar *dirname, 
					  gboolean overwrite, gboolean skip_new,
					  GList *selected_files)
{
  gchar *error_msg = NULL, *str, *base;
  guint8 filetype;


  /* Get base name */
  base = g_path_get_basename(file);

  /* Use status bar to indicate what we're doing */
  str = g_strdup_printf( _("%s: Extracting ..."), base);
  lintar_push_statusbar(str);
  g_free(str);
  
  /* Check if file exists AND IT *IS a directory */
  if (file==NULL) {
	error_msg = g_strdup_printf(_("Open an archive first"));	
  } else if (g_file_test( dirname, G_FILE_TEST_EXISTS)!=TRUE) {
	error_msg = g_strdup_printf( _("Could not find directory: %s"), dirname);	
  } else if (g_file_test( dirname, G_FILE_TEST_IS_DIR)!=TRUE) {
	error_msg = g_strdup_printf( _("%s is *NOT* a directory"), dirname);
  } else {
	
	/* Extract archive depending file name */
	filetype = lintar_get_compress_file_type(file);
	switch (filetype) {
	
	case LINTAR_TYPE_UNKNOWN:
	  error_msg = g_strdup_printf( _("Unknown archive format for %s"), file);
	  break;
	case LINTAR_TYPE_Z:
	case LINTAR_TYPE_GZ:
	case LINTAR_TYPE_BZ2:
	  extract_archive_single( file, dirname, filetype, overwrite, &error_msg );
	  break;
	case LINTAR_TYPE_TAR:
	  extract_archive_tar( file, dirname, "", overwrite, selected_files, &error_msg );
	break;
	case LINTAR_TYPE_TAR_Z:
	  extract_archive_tar( file, dirname, "Z", overwrite, selected_files, &error_msg );
	  break;
	case LINTAR_TYPE_TAR_GZ:
	  extract_archive_tar( file, dirname, "z", overwrite, selected_files, &error_msg );
	  break;
	case LINTAR_TYPE_TAR_BZ2:
	  extract_archive_tar( file, dirname, "j", overwrite, selected_files, &error_msg );
	  break;
	case LINTAR_TYPE_ZIP:
	  extract_archive_zip( file, dirname, overwrite, skip_new, selected_files, &error_msg );
	  break;
	default:
	  error_msg = g_strdup_printf( _("Unknown archive format for %s"), file);
	  break;
	}

  }
  
  if ( error_msg!=NULL ) {
		
	/* Report the error */
	lintar_report_error(error_msg);
	g_free(error_msg);

  }	else {
	
	str = g_strdup_printf( _("%s: Extracted."), base);
	lintar_push_statusbar(str);
	g_free(str);
	
  }
  
  /* Set progress bar */
  lintar_set_progressbar( 0.0 );
  
  /* Free all things */
  g_free(base);

}







