#include "prog_single.h"




GtkTreeModel *create_model_single( GtkWidget *window, 
								   gchar *file_name,
								   guint8 compresstype,
								   gboolean view,
								   gint *out_nfiles,
								   gint *out_tsize,
								   gchar **error_msg)
{
  gchar *program, *str, *tmp_name, *tmp_uncompressed, *p, *base, *ratio;
  gint size, nfiles=0, tsize=0, csize, r;
  GtkTreeStore *model=NULL;
  GtkTreeIter iter;
  GdkPixbuf *pixbuf;
  struct stat st_stat;

  /* Initialise nfiles and tsize */
  (*out_nfiles) = 0;
  (*out_tsize)  = 0;
  (*error_msg) = NULL;

  /* Get program to use */
  switch (compresstype) {
  case LINTAR_TYPE_BZ2:
	program = lintar_get_prog_bzip2();
	if ( program[0]=='\0' ) {
	  (*error_msg) = g_strdup(_("BZIP2 archives not supported."));
	}
	break;
  case LINTAR_TYPE_GZ:
	program = lintar_get_prog_gzip();
	if ( program[0]=='\0' ) {
	  (*error_msg) = g_strdup(_("GZIP archives not supported."));
	}
	break;
  case LINTAR_TYPE_Z:
	program = lintar_get_prog_compress();
	if ( program[0]=='\0' ) {
	  (*error_msg) = g_strdup(_("COMPRESS archives not supported."));
	}
	break;
  default:
	(*error_msg) = g_strdup(_("Unknown compress type"));
	break;
  }
  
  if ( (*error_msg)!=NULL ) {
	return(NULL);
  }

  /* Get temporal name */
  str = g_path_get_basename(file_name);
  tmp_name = g_strdup_printf("%s/%s", lintar_get_tmp_dir(), str);
  tmp_uncompressed = g_strdup(tmp_name);
  p = strrchr(tmp_uncompressed, '.');  if (p!=NULL) { (*p)='\0'; }
  g_free(str);

  /* Extract file into temporal directory */
  extract_archive_single( file_name, NULL, compresstype, TRUE, error_msg);

  /* Read file data */
  stat((const char *)tmp_uncompressed, &st_stat);
  size = (gint)st_stat.st_size;
  stat((const char *)file_name, &st_stat);
  csize = (gint)st_stat.st_size;
  r = (gint) (((gdouble)csize)/((gdouble)size)*100.0);
  if (r>100) {
	ratio = g_strdup("100%%");
  } else {
	ratio = g_strdup_printf("%02d%%", r);
  }

  /* get basename */
  base = g_path_get_basename(tmp_uncompressed); 

  /* view doesn't matter */
  /* create tree store */
  model = gtk_tree_store_new( NUM_COLUMNS, 
							  G_TYPE_STRING,    /* NAME_COLUMN            */
							  G_TYPE_STRING,    /* PATH_COLUMN            */
							  G_TYPE_STRING,    /* FULLNAME_COLUMN        */ 
							  G_TYPE_STRING,    /* PERMISSIONS_COLUMN     */
							  G_TYPE_STRING,    /* OWN_COLUMN             */
							  G_TYPE_INT,       /* SIZE_COLUMN            */
							  G_TYPE_STRING,    /* SIZE_COMPRESSED_COLUMN */
							  G_TYPE_STRING,    /* DATE_COLUMN            */
							  G_TYPE_STRING,    /* LINK_COLUMN            */
							  GDK_TYPE_PIXBUF); /* ICON_COLUMN            */

  pixbuf = lintar_get_pixbuf_from_file( window, 0, tmp_uncompressed);
  gtk_tree_store_append(model, &iter, NULL);
  gtk_tree_store_set( model, &iter, 
					  NAME_COLUMN, base,  
					  FULLNAME_COLUMN, base,
					  SIZE_COLUMN, size, 
					  SIZE_COMPRESSED_COLUMN, ratio,
					  ICON_COLUMN, pixbuf, -1);
  nfiles = 1;
  tsize += size;

  /* Free allocated strings */
  g_free(base); g_free(tmp_name); g_free(tmp_uncompressed); 
  g_free(ratio);

  /* return variables */
  (*out_nfiles) = nfiles;
  (*out_tsize) = tsize;
  return(GTK_TREE_MODEL(model));

}



void add_columns_single(GtkTreeView *treeview, gboolean view)
{
  GtkCellRenderer *renderer;
  GtkTreeViewColumn *column;

  /* Name column */
  column = gtk_tree_view_column_new ();
  gtk_tree_view_column_set_title (column, _("Name") );
  renderer = gtk_cell_renderer_pixbuf_new ();
  gtk_tree_view_column_pack_start (column, renderer, FALSE);
  gtk_tree_view_column_set_attributes(column, renderer, 
									  "pixbuf", ICON_COLUMN, NULL);
  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_column_pack_start (column, renderer, TRUE);
  gtk_tree_view_column_set_attributes(column, renderer, 
									  "text", NAME_COLUMN, NULL);
  gtk_tree_view_column_set_sort_column_id (column, NAME_COLUMN);
  gtk_tree_view_column_set_resizable( column, TRUE);
  gtk_tree_view_append_column( GTK_TREE_VIEW(treeview), column);
  
  /* Size */
  renderer = gtk_cell_renderer_text_new();
  g_object_set (G_OBJECT (renderer), "xalign", 0.0, NULL);
  column = gtk_tree_view_column_new_with_attributes( _("Size"), renderer, 
	 												 "text", SIZE_COLUMN, NULL);
  gtk_tree_view_column_set_sort_column_id (column, SIZE_COLUMN);
  gtk_tree_view_column_set_resizable( column, TRUE);
  gtk_tree_view_append_column( GTK_TREE_VIEW(treeview), column);
  
  /* Size Compressed */
  renderer = gtk_cell_renderer_text_new();
  g_object_set (G_OBJECT (renderer), "xalign", 0.0, NULL);
  column = gtk_tree_view_column_new_with_attributes( _("Ratio"), renderer, 
	 												 "text", SIZE_COMPRESSED_COLUMN, NULL);
  gtk_tree_view_column_set_sort_column_id (column, SIZE_COMPRESSED_COLUMN);
  gtk_tree_view_column_set_resizable( column, TRUE);
  gtk_tree_view_append_column( GTK_TREE_VIEW(treeview), column);

}




void extract_archive_single( gchar *filename, gchar *dir, guint8 compresstype, 
							 gboolean overwrite, gchar **error_msg)
{
  gchar **argv, *standard_output, *standard_error, *program=NULL, *name_uncompressed;
  gchar *tmp_name, *tmp_uncompressed, *str, *p;
  gint exit_status = 0, i;
  gboolean spawn;
  GError *gerror = NULL;
  
  (*error_msg) = NULL;
  
  /* Get program to use */
  switch (compresstype) {
  case LINTAR_TYPE_BZ2:
	program = lintar_get_prog_bzip2();
	if ( program[0]=='\0' ) {
	  (*error_msg) = g_strdup(_("BZIP2 archives not supported."));
	}
	break;
  case LINTAR_TYPE_GZ:
	program = lintar_get_prog_gzip();
	if ( program[0]=='\0' ) {
	  (*error_msg) = g_strdup(_("GZIP archives not supported."));
	}
	break;
  case LINTAR_TYPE_Z:
	program = lintar_get_prog_compress();
	if ( program[0]=='\0' ) {
	  (*error_msg) = g_strdup(_("COMPRESS archives not supported."));
	}
	break;
  default:
	(*error_msg) = g_strdup(_("Unknown compress type"));
	break;
  }

  /* Get future uncompressed name ! */
  str = g_path_get_basename(filename);
  if (dir!=NULL) {
	name_uncompressed = g_strdup_printf("%s/%s", dir, str);
  } else {
	name_uncompressed = g_strdup_printf("%s/%s", lintar_get_tmp_dir(), str);
  }
  p = strrchr(name_uncompressed, '.'); if (p!=NULL) { (*p)='\0'; }
  g_free(str);

  /* Check if we're going to overwrite, it'll save time! */
  if (!overwrite) {
	if (dir!=NULL) {
	  if (g_file_test( name_uncompressed, G_FILE_TEST_EXISTS)) {
		(*error_msg) = g_strdup_printf(_("File %s already exists."), name_uncompressed);
	  }
	}
  }

  /* 
	 Let's uncompress the file, the process is:
	 - Make a link to the temporal folder.
	 - Uncompress the file on the temporal folder
	 - Finally, copy it to the final location
  */
  if ( (*error_msg)!=NULL ) {
	
	/* Do nothing else, we found an error */

  } else {

	/* Get temporal name */
	str = g_path_get_basename(filename);
	tmp_name = g_strdup_printf("%s/%s", lintar_get_tmp_dir(), str);
	tmp_uncompressed = g_strdup(tmp_name);
	p = strrchr(tmp_uncompressed, '.');  if (p!=NULL) { (*p)='\0'; }
	g_free(str);
	
	/* Make sure tmp name is not use */
	if (g_file_test(tmp_name,G_FILE_TEST_EXISTS)) {
	  unlink(tmp_name);
	}
	
	/* Copy original file */
	copy_file( filename, tmp_name, TRUE);

	/* Uncompress temporal file */
	i=0; argv = (gchar **)g_malloc( (gulong)(10)*sizeof(gchar*) );
	argv[i++] = g_strdup(program); 
	argv[i++] = g_strdup("-d");
	argv[i++] = g_strdup("-f");
	argv[i++] = g_strdup(tmp_name);
	argv[i] = NULL;
	spawn = thread_g_spawn_sync( NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, 
								 &standard_output, &standard_error, &exit_status,
								 &gerror);	
	g_strfreev(argv);
	
	/* Check for errors */
	if (standard_error!=NULL) { 
	  if (standard_error[0]!='\0') { (*error_msg) = g_strdup(standard_error); }
	}
   	if ( ((*error_msg)==NULL) && (exit_status!=0) ) {
	  (*error_msg) = g_strdup_printf(_("An error ocurred extracting file %s"), filename);
	}
	if (spawn==FALSE) {
	  if (gerror->message!=NULL) {
		(*error_msg) = g_strdup(gerror->message);
	  } else {
		(*error_msg) = g_strdup_printf(_("An error ocurred extracting file %s"), filename);
	  }
	}
	
	/* Copy file to final location if everything went OK */
	if ( (*error_msg)==NULL ) {
	  if (dir!=NULL) {
		if (dir!=lintar_get_tmp_dir()) {
		  copy_file( tmp_uncompressed, name_uncompressed,TRUE);
		}
	  }
	}
	
	/* Free memory */
	if (standard_output!=NULL) {
	  g_free(standard_output);
	}
	if (standard_error!=NULL) {
	  g_free(standard_error);
	}
	g_free(tmp_name);
	g_free(tmp_uncompressed);
	
  }


  g_free(name_uncompressed);

}






