#include "prog_tar.h"




GtkTreeModel *create_model_tar( GtkWidget *window, 
								gchar *file_name,
								gchar *compress,
								gboolean view,
								gint *out_nfiles,
								gint *out_tsize,
								gchar **error_msg)
{
  gchar **argv;
  GtkTreeStore *model=NULL;
  GtkTreeIter iter, last_iter;
  gchar *standard_output=NULL, *standard_error=NULL, *s=NULL;
  gint exit_status, i;
  gboolean done=FALSE, last_iter_valid=FALSE, spawn;
  guint8 type;
  ST_comp_data data;
  gchar d[6][LINTAR_MAX_LENGTH], *gp, **strlst, *pc;
  gchar *name, *date, *base, *dir, *link, *permissions; //*size_compressed;
  gchar *fullname, *owner;
  gint size, nlines, nfiles=0, tsize=0, line;
  GdkPixbuf *pixbuf;
  GError *gerror = NULL;

  /* Initialise nfiles and tsize */
  (*out_nfiles) = 0;
  (*out_tsize)  = 0;
  (*error_msg) = NULL;

  /* Check if any program is missing */
  if ( lintar_get_prog_tar()[0]=='\0' ) {
	(*error_msg)  = g_strdup(_("TAR archives not supported."));
  }
  
  switch (compress[0]) {
  case 'Z':
	if ( lintar_get_prog_compress()[0]=='\0' ) {
	  (*error_msg) = g_strdup(_("COMPRESS archives not supported."));
	}
	break;
  case 'z':
	if ( lintar_get_prog_gzip()[0]=='\0' ) {
	  (*error_msg) = g_strdup(_("GZIP archives not supported."));
	}
	break;
  case 'j':
	if ( lintar_get_prog_bzip2()[0]=='\0' ) {
	  (*error_msg) = g_strdup(_("BZIP2 archives not supported."));
	}
	break;
  default:
	break;
  }
  
  if ( (*error_msg)!=NULL ) {
	return(NULL);
  }

  /* create argv */
  i = 0;
  argv = (gchar **)g_malloc( (gulong)(9*sizeof(char*)));
  argv[i++] = g_strdup(lintar_get_prog_tar()); 
  argv[i++] = g_strdup_printf("t%svf", compress);
  argv[i++] = g_strdup(file_name); 
  argv[i]   = NULL;
  
    
  /* Call tar to see contents of the file */
  spawn = thread_g_spawn_sync( NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, 
							   &standard_output, &standard_error, &exit_status,
							   &gerror);

  g_strfreev(argv);
  
  /* Check for errors in tar call */
  if (standard_error!=NULL) {
	if (standard_error[0]!='\0') {
	  (*error_msg) = g_strdup(standard_error);
	  g_free(standard_error); g_free(standard_output);
	  return(NULL); 
	}
  }
  if (spawn==FALSE) {
	if (gerror->message!=NULL) {
	  (*error_msg) = g_strdup(gerror->message);
	} else {
	  (*error_msg) = g_strdup_printf(_("An error ocurred opening file %s"), file_name);
	}
	return(NULL);
  }

  /* create tree store */
  model = gtk_tree_store_new( NUM_COLUMNS, 
							  G_TYPE_STRING,    /* NAME_COLUMN            */
							  G_TYPE_STRING,    /* PATH_COLUMN            */
							  G_TYPE_STRING,    /* FULLNAME_COLUMN        */ 
							  G_TYPE_STRING,    /* PERMISSIONS_COLUMN     */
							  G_TYPE_STRING,    /* OWN_COLUMN             */
							  G_TYPE_INT,       /* SIZE_COLUMN            */
							  G_TYPE_STRING,    /* SIZE_COMPRESSED_COLUMN */
							  G_TYPE_STRING,    /* DATE_COLUMN            */
							  G_TYPE_STRING,    /* LINK_COLUMN            */
							  GDK_TYPE_PIXBUF); /* ICON_COLUMN            */

  /* Read output of tar */
  
  /* Count number of lines */
  pc = standard_output;
  nlines = -1;
  while ( pc!=NULL ) {
	nlines++;
	pc = index(pc+1,'\n');
  }

  /* Init some data */
  gp = standard_output;
  line = nfiles = tsize = 0;
  done = FALSE;

  while(!done) {

	type = 0;
	
	/* We're on line 'line' out of 'nlines' */
	lintar_set_progressbar( ((gdouble)line) / ((gdouble)nlines) );

	/* Read standard output */
	/* -rw-r--r-- owner/group     13001 1997-07-12 15:29 myfilename */
	/* 0          1               2     3          4     5 */
	s = g_strdup_printf( "%%%ds %%%ds %%%ds %%%ds %%%ds %%%ds", 
						 LINTAR_MAX_LENGTH, LINTAR_MAX_LENGTH, LINTAR_MAX_LENGTH, 
						 LINTAR_MAX_LENGTH, LINTAR_MAX_LENGTH, LINTAR_MAX_LENGTH);
	sscanf( gp, s, d[0],d[1],d[2],d[3],d[4],d[5]);
	g_free(s);
	
	if (strlen(d[3])==3) { 
	  /* -rw-r--r-- owner/group     13001 Dec 12 15:29 1997 myfilename */
	  /* 0          1               2     3   4  5     6    7          */
	  char a[4][LINTAR_MAX_LENGTH];

	  s = g_strdup_printf( "%%%ds %%%ds %%%ds %%%ds %%%ds %%%ds %%%ds %%%ds", 
						   LINTAR_MAX_LENGTH, LINTAR_MAX_LENGTH, LINTAR_MAX_LENGTH, 
						   LINTAR_MAX_LENGTH, LINTAR_MAX_LENGTH, LINTAR_MAX_LENGTH,
						   LINTAR_MAX_LENGTH, LINTAR_MAX_LENGTH);
	  sscanf( gp, s, d[0],d[1],d[2],a[0],a[1],a[2],a[3],d[5]);
	  g_free(s);

	  date = g_strdup_printf( "%s %s %s %s", a[0], a[1], a[2], a[3]); 

	} else {
	  date = g_strdup_printf( "%s %s", d[3], d[4]);
	}

	/* Permissions */
	permissions = g_strdup(d[0]);

	/* Owner/Group */
	owner = g_strdup(d[1]);

	/* Size */
	//size_compressed = g_strdup(d[5]);  
	size = (gint) atoi(d[2]);

	/* Copy fullname to variable 'name' */
	lintar_get_full_filename( gp, d[5], &name);
	
	/* Check symbolic links (and remove them from name) */
	lintar_extract_symbolic_link(gp, name, &link, &type);

	/* Copy full filename */
	fullname = g_strdup(name);

	/* Remove last '/' in the case of directories */
	lintar_remove_directory( permissions, name, &type);
	
	/* get basename and path */
	base = g_path_get_basename(name); 
	dir  = g_path_get_dirname(name); 
	

	if (!view) {

	  //LINTAR_VIEW_LIST:

	  /* Copy data to tree only if not a directory */
	  if (type!=LINTAR_TYPE_FOLDER) {
		
		pixbuf = lintar_get_pixbuf_from_file( window, type, fullname);
		gtk_tree_store_append(model, &iter, NULL);
		gtk_tree_store_set( model, &iter, 
							NAME_COLUMN, base,  
							PATH_COLUMN, dir,
							FULLNAME_COLUMN, fullname,
							PERMISSIONS_COLUMN, permissions, 
							OWN_COLUMN, owner,
							SIZE_COLUMN, size, 
							//SIZE_COMPRESSED_COLUMN, size_compressed,
							DATE_COLUMN, date, 
							LINK_COLUMN, link, 
							ICON_COLUMN, pixbuf, -1);
		nfiles++;
		tsize += size;
	  }

	} else {

	  //LINTAR_VIEW_TREE:

	  /* find tree path for this node */
	  strlst = g_strsplit( name, "/", G_MAXINT);
	  last_iter_valid = FALSE; i = 0;

	  while( strlst[i]!=NULL) {
		
		data.name = strlst[i];
		data.iter_valid = FALSE;
		data.depth = i;
		
		/* Find parent node of this */
		gtk_tree_model_foreach( GTK_TREE_MODEL(model), &tree_cmp_func, &data);

		if (data.iter_valid) {
		  
		  last_iter = data.iter;
		  last_iter_valid = TRUE;
		  
		} else {
		  
		  /* create node */
		  
		  if (last_iter_valid) {
			gtk_tree_store_append(model, &iter, &last_iter);
		  } else {
			/* include element as root */
			gtk_tree_store_append(model, &iter, NULL);
		  }

		  
		  if (strlst[i+1]==NULL) {

			/* It's a filename */

			/* Create Icon */
			pixbuf = lintar_get_pixbuf_from_file( window, type, fullname);

			/* Count number of files and total size */
			nfiles++;
			tsize += size;

			gtk_tree_store_set( model, &iter, 
								NAME_COLUMN, strlst[i],
								PATH_COLUMN, dir,
								FULLNAME_COLUMN, fullname,
								PERMISSIONS_COLUMN, permissions, 
								OWN_COLUMN, owner,
								SIZE_COLUMN, size, 
								//SIZE_COMPRESSED_COLUMN, size_compressed,
								DATE_COLUMN, date, 
								LINK_COLUMN, link, 
								ICON_COLUMN, pixbuf, -1);

		  } else { 
		  
			/* It's a directory in a filename */

			/* Create folder icon */
			pixbuf = gtk_widget_render_icon( window, GTK_STOCK_OPEN, 
											 GTK_ICON_SIZE_MENU, NULL);
			
			/* Do not count files or size */

			gtk_tree_store_set( model, &iter, 
								NAME_COLUMN, strlst[i],
								//PATH_COLUMN, dir,
								//FULLNAME_COLUMN, fullname,
								//PERMISSIONS_COLUMN, permissions, 
								//OWN_COLUMN, owner,
								//SIZE_COLUMN, size, 
								//SIZE_COMPRESSED_COLUMN, size_compressed,
								//DATE_COLUMN, date, 
								//LINK_COLUMN, link, 
								ICON_COLUMN, pixbuf, -1);

		  }

		  
		  /* set last included element as valid ref */
		  last_iter = iter;
		  last_iter_valid = TRUE;

		}
		
		i++;
		
	  }
	  
	  g_strfreev(strlst);
	  
	}
	  
	/* Move to the next line */
	gp = strchr( gp, '\n');
	if (gp[1]=='\000') {
	  done = TRUE;
	} else {
	  gp++;
	}
	
	line++;

	/* Free allocated strings */
	g_free(name); g_free(fullname); g_free(date); g_free(link); g_free(base); g_free(dir);
	g_free(permissions); g_free(owner);

  } /* while (!done) */
  
  (*out_nfiles) = nfiles;
  (*out_tsize) = tsize;
  return(GTK_TREE_MODEL(model));

}




void add_columns_tar(GtkTreeView *treeview, gboolean view)
{
  GtkCellRenderer *renderer;
  GtkTreeViewColumn *column;

  /* Name column */
  column = gtk_tree_view_column_new ();
  gtk_tree_view_column_set_title (column, _("Name") );
  renderer = gtk_cell_renderer_pixbuf_new ();
  gtk_tree_view_column_pack_start (column, renderer, FALSE);
  gtk_tree_view_column_set_attributes(column, renderer, 
									  "pixbuf", ICON_COLUMN, NULL);
  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_column_pack_start (column, renderer, TRUE);
  gtk_tree_view_column_set_attributes(column, renderer, 
									  "text", NAME_COLUMN, NULL);
  gtk_tree_view_column_set_sort_column_id (column, NAME_COLUMN);
  gtk_tree_view_column_set_resizable( column, TRUE);
  gtk_tree_view_append_column( GTK_TREE_VIEW(treeview), column);
  
  /* Permissions column */
  renderer = gtk_cell_renderer_text_new();
  g_object_set (G_OBJECT (renderer), "xalign", 0.0, NULL);
  column = gtk_tree_view_column_new_with_attributes( _("Permissions"), renderer, 
													 "text", 
													 PERMISSIONS_COLUMN, 
													 NULL);
  gtk_tree_view_column_set_sort_column_id (column, PERMISSIONS_COLUMN);
  gtk_tree_view_column_set_resizable( column, TRUE);
  gtk_tree_view_append_column( GTK_TREE_VIEW(treeview), column);

  /* Owner */
  renderer = gtk_cell_renderer_text_new();
  g_object_set (G_OBJECT (renderer), "xalign", 0.0, NULL);
  column = gtk_tree_view_column_new_with_attributes( _("Owner/Group"), renderer, 
													 "text", OWN_COLUMN, NULL);
  gtk_tree_view_column_set_sort_column_id (column, OWN_COLUMN);
  gtk_tree_view_column_set_resizable( column, TRUE);
  gtk_tree_view_append_column( GTK_TREE_VIEW(treeview), column);

  /* Size */
  renderer = gtk_cell_renderer_text_new();
  g_object_set (G_OBJECT (renderer), "xalign", 0.0, NULL);
  column = gtk_tree_view_column_new_with_attributes( _("Size"), renderer, 
	 												 "text", SIZE_COLUMN, NULL);
  gtk_tree_view_column_set_sort_column_id (column, SIZE_COLUMN);
  gtk_tree_view_column_set_resizable( column, TRUE);
  gtk_tree_view_append_column( GTK_TREE_VIEW(treeview), column);
  
  /* Date */
  renderer = gtk_cell_renderer_text_new();
  g_object_set (G_OBJECT (renderer), "xalign", 0.0, NULL);
  column = gtk_tree_view_column_new_with_attributes( _("Date"), renderer, 
	 												 "text", DATE_COLUMN, NULL);
  gtk_tree_view_column_set_sort_column_id (column, DATE_COLUMN);
  gtk_tree_view_column_set_resizable( column, TRUE);
  gtk_tree_view_append_column( GTK_TREE_VIEW(treeview), column);

  /* Date */
  renderer = gtk_cell_renderer_text_new();
  g_object_set (G_OBJECT (renderer), "xalign", 0.0, NULL);
  column = gtk_tree_view_column_new_with_attributes( _("Link"), renderer, 
	 												 "text", LINK_COLUMN, NULL);
  gtk_tree_view_column_set_sort_column_id (column, LINK_COLUMN);
  gtk_tree_view_column_set_resizable( column, TRUE);
  gtk_tree_view_append_column( GTK_TREE_VIEW(treeview), column);

  if (view==FALSE) {
	/* PATH */
	renderer = gtk_cell_renderer_text_new();
	g_object_set (G_OBJECT (renderer), "xalign", 0.0, NULL);
	column = gtk_tree_view_column_new_with_attributes( _("Path"), renderer, 
													   "text", PATH_COLUMN, NULL);
	gtk_tree_view_column_set_sort_column_id (column, PATH_COLUMN);
	gtk_tree_view_column_set_resizable( column, TRUE);
	gtk_tree_view_append_column( GTK_TREE_VIEW(treeview), column);
  }

}


void extract_archive_tar( gchar *filename, gchar *dir, gchar *compress, 
						  gboolean overwrite, GList *selected_files, 
						  gchar **error_msg)
{
  GList *l;
  gchar **argv, *standard_output, *standard_error;
  gint exit_status = 0, i;
  gboolean spawn;
  GError *gerror = NULL;

  (*error_msg) = NULL;

  if ( lintar_get_prog_tar()[0]=='\0' ) {
	(*error_msg) = g_strdup(_("ZIP archives not supported."));
  } else {

	/* Extract file into directory dir */
	if (selected_files!=NULL) {
	  l = g_list_first(selected_files);
	  argv = (gchar **)g_malloc( (gulong)(10+g_list_length(l))*sizeof(gchar*) );
	} else {
	  l = NULL;
	  argv = (gchar **)g_malloc( (gulong)(10)*sizeof(gchar*) );
	}

	i = 0;
	argv[i++] = g_strdup(lintar_get_prog_tar()); 
	argv[i++] = g_strdup_printf("x%sf", compress);
	argv[i++] = g_strdup(filename); 
	argv[i++] = g_strdup("--directory");
	argv[i++] = g_strdup(dir); 
	if (overwrite) { 
	  argv[i++] = g_strdup("--overwrite");
	} else {
	  argv[i++] = g_strdup("--keep-old-files");
	}
	while (l!=NULL) {
	  argv[i++] = g_strdup(l->data);
	  l = g_list_next(l);
	}
	argv[i] = NULL;
	
	/* Call tar to see contents of the file */
	spawn = thread_g_spawn_sync( NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, 
								 &standard_output, &standard_error, &exit_status,
								 &gerror);

	g_strfreev(argv);

	if (standard_error!=NULL) { 
	  if (standard_error[0]!='\000') {
		if (overwrite==FALSE) {
		  /* FIXME: Tar complains when overwritting files so do nothing */
		} else {
		  (*error_msg) = g_strdup(standard_error);		  
		}
	  }
	}
	
	if ( ((*error_msg)==NULL) && (exit_status!=0) ) {
	  (*error_msg) = g_strdup_printf(_("An error ocurred extracting file %s"), filename);
	}
	if (spawn==FALSE) {
	  if (gerror->message!=NULL) {
		(*error_msg) = g_strdup(gerror->message);
	  } else {
		(*error_msg) = g_strdup_printf(_("An error ocurred extracting file %s"), filename);
	  }
	}
	
	/* Free memory */
	if (standard_output!=NULL) {
	  g_free(standard_output);
	}
	if (standard_error!=NULL) {
	  g_free(standard_error);
	}

  }

}






