#include "prog_tar.h"




GtkTreeModel *create_model_zip( GtkWidget *window, 
								gchar *file_name,
								gboolean view,
								gint *out_nfiles, gint *out_tsize,
								gchar **error_msg)
{
  GtkTreeStore *model;
  GtkTreeIter iter, last_iter;
  gchar *standard_output=NULL, *standard_error=NULL, *s=NULL;
  gint exit_status, i;
  gboolean done=FALSE, last_iter_valid=FALSE, spawn;
  guint8 type;
  ST_comp_data data;
  gchar d[10][LINTAR_MAX_LENGTH], *gp, *pc, **strlst, **argv;
  gchar *name, *date, *base, *dir, *link, *permissions, *size_compressed;
  gchar *fullname; //*owner;
  gint size, nfiles, tsize, nlines, line;
  GdkPixbuf *pixbuf;
  GError *gerror = NULL;

  if ( lintar_get_prog_unzip()[0]=='\0' ) {
	(*error_msg) = g_strdup(_("ZIP archives not supported."));
	return(NULL);
  }
  
  /* create arg */
  i = 0;
  argv = (gchar **)g_malloc( (gulong)(5*sizeof(char*)));
  argv[i++] = g_strdup(lintar_get_prog_unzip()); 
  argv[i++] = g_strdup("-Z");
  argv[i++] = g_strdup("-m");
  argv[i++] = g_strdup(file_name); 
  argv[i]   = NULL;
  
  /* Call tar to see contents of the file */
  spawn = thread_g_spawn_sync( NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, 
							   &standard_output, &standard_error, &exit_status,
							   &gerror);  //ignore errors

  g_strfreev(argv);
  
  /* Check for errors in zip call */
  if (standard_error!=NULL) {
	if (standard_error[0]!='\0') {
	  (*error_msg) = g_strdup(standard_error);
	  g_free(standard_error); g_free(standard_output);
	  return(NULL); 
	}
  }
  if (spawn==FALSE) {
	if (gerror->message!=NULL) {
	  (*error_msg) = g_strdup(gerror->message);
	} else {
	  (*error_msg) = g_strdup_printf(_("An error ocurred opening file %s"), file_name);
	}
	return(NULL);
  }
  
  /* Check for more errors */
  /* FIXME: Check output of unzip in other locales */
  if (g_ascii_strncasecmp( standard_output, "Archive:", 8)!=0) {
	(*error_msg) = g_strdup(_("Unzip output not supported!"));
	return(NULL);
  }
  
  /* create tree store */
  model = gtk_tree_store_new( NUM_COLUMNS, 
							  G_TYPE_STRING,    /* NAME_COLUMN            */
							  G_TYPE_STRING,    /* PATH_COLUMN            */
							  G_TYPE_STRING,    /* FULLNAME_COLUMN        */ 
							  G_TYPE_STRING,    /* PERMISSIONS_COLUMN     */
							  G_TYPE_STRING,    /* OWN_COLUMN             */
							  G_TYPE_INT,       /* SIZE_COLUMN            */
							  G_TYPE_STRING,    /* SIZE_COMPRESSED_COLUMN */
							  G_TYPE_STRING,    /* DATE_COLUMN            */
							  G_TYPE_STRING,    /* LINK_COLUMN            */
							  GDK_TYPE_PIXBUF); /* ICON_COLUMN            */
  
  /* Read output of zip */

  /* Count number of lines */
  pc = standard_output;
  nlines = -3;
  while ( pc!=NULL ) {
	nlines++;
	pc = index(pc+1,'\n');
  }

  /* Init some data */
  gp = standard_output;
  line = nfiles = tsize = 0;
  done = FALSE;

  /* First line not used */
  gp = strchr( gp, '\n') + 1;
  
  done = FALSE;
  while(!done) {
	
	type = 0;
	
	/* We're on line 'line' out of 'nlines' */
	lintar_set_progressbar( ((gdouble)line) / ((gdouble)nlines) );

	/* Read standard output */
	/* -rwxa--     2.2 ntf   395264 bx  3% defX 29-Mar-01 14:26 filename */
	/* 0           1   2     3      4   5  6    7         8     9        */
	s = g_strdup_printf( "%%%ds %%%ds %%%ds %%%ds %%%ds %%%ds %%%ds %%%ds %%%ds %%%ds", 
						 LINTAR_MAX_LENGTH, LINTAR_MAX_LENGTH, LINTAR_MAX_LENGTH, 
						 LINTAR_MAX_LENGTH, LINTAR_MAX_LENGTH, LINTAR_MAX_LENGTH,
						 LINTAR_MAX_LENGTH, LINTAR_MAX_LENGTH, LINTAR_MAX_LENGTH, 
						 LINTAR_MAX_LENGTH);
	sscanf( gp, s, d[0],d[1],d[2],d[3],d[4],d[5],d[6],d[7],d[8],d[9]);
	g_free(s);
   
	/* Permissions */
	permissions = g_strdup(d[0]);

	/* Compose date column */
	date = g_strdup_printf( "%s %s", d[7], d[8]);
	
	/* Owner/Group (no info on zip files) */
	//owner = g_strdup("");
 
	/* Size */
	size_compressed = g_strdup(d[5]);
	size = (gint) atoi(d[3]);
	
	/* Copy fullname to variable name */
	lintar_get_full_filename( gp, d[9], &name);
	
	/* Check symbolic links (and remove them from name) */
	lintar_extract_symbolic_link(gp, name, &link, &type);
	
	/* Copy full filename */
	fullname = g_strdup(name);
	
	/* Remove last '/' in the case of directories */
	lintar_remove_directory( permissions, name, &type);

	/* get basename and path */
	base = g_path_get_basename(name); 
	dir  = g_path_get_dirname(name); 
	

	if (!view) {

	  // LINTAR_VIEW_LIST:
	  
	  /* Copy data to tree only if not a directory */
	  if (type != LINTAR_TYPE_FOLDER) {
		
		pixbuf = lintar_get_pixbuf_from_file( window, type, fullname);
		gtk_tree_store_append(model, &iter, NULL);
		gtk_tree_store_set( model, &iter, 
							NAME_COLUMN, base,  
							PATH_COLUMN, dir,
							FULLNAME_COLUMN, fullname,
							PERMISSIONS_COLUMN, permissions, 
							//OWN_COLUMN, owner,
							SIZE_COLUMN, size, 
							SIZE_COMPRESSED_COLUMN, size_compressed,
							DATE_COLUMN, date, 
							LINK_COLUMN, link, 
							ICON_COLUMN, pixbuf, -1);
		nfiles++;
		tsize += size;
	  }

	} else {
	  
	  // LINTAR_VIEW_TREE:

	  /* find tree path for this node */
	  strlst = g_strsplit( name, "/", G_MAXINT);
	  last_iter_valid = FALSE; i = 0;

	  while( strlst[i]!=NULL) {
		
		data.name = strlst[i];
		data.iter_valid = FALSE;
		data.depth = i;
		  
		/* Find parent node of this */
		gtk_tree_model_foreach( GTK_TREE_MODEL(model), &tree_cmp_func, &data);
		
		if (data.iter_valid) {

		  last_iter = data.iter;
		  last_iter_valid = TRUE;

		} else {
			
		  /* create node */

		  if (last_iter_valid) {
			gtk_tree_store_append(model, &iter, &last_iter);
		  } else {
			/* include element as root */
			gtk_tree_store_append(model, &iter, NULL);
		  }
		  
		  if (strlst[i+1]==NULL) {

			/* It's a filename */
			
			/* Create Icon */
			pixbuf = lintar_get_pixbuf_from_file( window, type, fullname);
			
			/* Count number of files and total size */
			nfiles++;
			tsize += size;
			
			gtk_tree_store_set( model, &iter, 
								NAME_COLUMN, strlst[i],  
								//PATH_COLUMN, dir,
								FULLNAME_COLUMN, fullname,
								PERMISSIONS_COLUMN, permissions, 
								//OWN_COLUMN, owner,
								SIZE_COLUMN, size, 
								SIZE_COMPRESSED_COLUMN, size_compressed,
								DATE_COLUMN, date, 
								LINK_COLUMN, link, 
								ICON_COLUMN, pixbuf, -1);

		  } else {
	
			/* It's a directory in a filename */

			/* Create folder icon */
			pixbuf = gtk_widget_render_icon( window, GTK_STOCK_OPEN, 
											 GTK_ICON_SIZE_MENU, NULL);
			
			/* Do not count files or size */

			gtk_tree_store_set( model, &iter, 
								NAME_COLUMN, strlst[i],  
								//PATH_COLUMN, dir,
								//FULLNAME_COLUMN, fullname,
								//PERMISSIONS_COLUMN, permissions, 
								//OWN_COLUMN, owner,
								//SIZE_COLUMN, size, 
								//SIZE_COMPRESSED_COLUMN, size_compressed,
								//DATE_COLUMN, date, 
								//LINK_COLUMN, link, 
								ICON_COLUMN, pixbuf, -1);

		  }
		  
		  
		  /* set last included element as valid ref */
		  last_iter = iter;
		  last_iter_valid = TRUE;

		}
		
		i++;
		
	  }
	  
	  g_strfreev(strlst);

	}
	  
	/* Move to the next line */
	gp = strchr( gp, '\n');
	if (gp[1]=='\000') {
	  done = TRUE;
	} else {
	  gp++;
	}

	/* Check final line */
	if (done==FALSE) {
	  pc = strchr( gp, '\n');
	  if (pc[1]=='\0') {
		done = TRUE;
	  }
	}
	
	line++;

	/* Free allocated strings */
	g_free(name); g_free(fullname); g_free(date); g_free(link); g_free(base); g_free(dir);
	g_free(permissions); g_free(size_compressed);

  } /* while (!done) */

  (*out_nfiles) = nfiles;
  (*out_tsize) = tsize;  
  return( GTK_TREE_MODEL(model) );

}




void add_columns_zip(GtkTreeView *treeview, gboolean view)
{
  GtkCellRenderer *renderer;
  GtkTreeViewColumn *column;

  /* Name column */
  column = gtk_tree_view_column_new ();
  gtk_tree_view_column_set_title (column, _("Name") );
  renderer = gtk_cell_renderer_pixbuf_new ();
  gtk_tree_view_column_pack_start (column, renderer, FALSE);
  gtk_tree_view_column_set_attributes(column, renderer, 
									  "pixbuf", ICON_COLUMN, NULL);
  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_column_pack_start (column, renderer, TRUE);
  gtk_tree_view_column_set_attributes(column, renderer, 
									  "text", NAME_COLUMN, NULL);
  gtk_tree_view_column_set_sort_column_id (column, NAME_COLUMN);
  gtk_tree_view_column_set_resizable( column, TRUE);
  gtk_tree_view_append_column( GTK_TREE_VIEW(treeview), column);
  
  /* Permissions column */
  renderer = gtk_cell_renderer_text_new();
  g_object_set (G_OBJECT (renderer), "xalign", 0.0, NULL);
  column = gtk_tree_view_column_new_with_attributes( _("Permissions"), renderer, 
													 "text", 
													 PERMISSIONS_COLUMN, 
													 NULL);
  gtk_tree_view_column_set_sort_column_id (column, PERMISSIONS_COLUMN);
  gtk_tree_view_column_set_resizable( column, TRUE);
  gtk_tree_view_append_column( GTK_TREE_VIEW(treeview), column);
  
  /* Size */
  renderer = gtk_cell_renderer_text_new();
  g_object_set (G_OBJECT (renderer), "xalign", 0.0, NULL);
  column = gtk_tree_view_column_new_with_attributes( _("Size"), renderer, 
	 												 "text", SIZE_COLUMN, NULL);
  gtk_tree_view_column_set_sort_column_id (column, SIZE_COLUMN);
  gtk_tree_view_column_set_resizable( column, TRUE);
  gtk_tree_view_append_column( GTK_TREE_VIEW(treeview), column);
  
  /* Size Compressed */
  renderer = gtk_cell_renderer_text_new();
  g_object_set (G_OBJECT (renderer), "xalign", 0.0, NULL);
  column = gtk_tree_view_column_new_with_attributes( _("Ratio"), renderer, 
													 "text", SIZE_COMPRESSED_COLUMN, NULL);
  gtk_tree_view_column_set_sort_column_id (column, SIZE_COMPRESSED_COLUMN);
  gtk_tree_view_column_set_resizable( column, TRUE);
  gtk_tree_view_append_column( GTK_TREE_VIEW(treeview), column);

  /* Date */
  renderer = gtk_cell_renderer_text_new();
  g_object_set (G_OBJECT (renderer), "xalign", 0.0, NULL);
  column = gtk_tree_view_column_new_with_attributes( _("Date"), renderer, 
	 												 "text", DATE_COLUMN, NULL);
  gtk_tree_view_column_set_sort_column_id (column, DATE_COLUMN);
  gtk_tree_view_column_set_resizable( column, TRUE);
  gtk_tree_view_append_column( GTK_TREE_VIEW(treeview), column);
  
  if (view==FALSE) {
	/* PATH */
	renderer = gtk_cell_renderer_text_new();
	g_object_set (G_OBJECT (renderer), "xalign", 0.0, NULL);
	column = gtk_tree_view_column_new_with_attributes( _("Path"), renderer, 
													   "text", PATH_COLUMN, NULL);
	gtk_tree_view_column_set_sort_column_id (column, PATH_COLUMN);
	gtk_tree_view_column_set_resizable( column, TRUE);
	gtk_tree_view_append_column( GTK_TREE_VIEW(treeview), column);
  }
  
}


void extract_archive_zip( gchar *filename, gchar *dir, 
						  gboolean overwrite, gboolean skip_new, 
						  GList *selected_files, gchar **error_msg) 
{
  GList *l;
  gchar **argv, *standard_output, *standard_error;
  gint exit_status = 0, i;
  gboolean spawn;
  GError *gerror = NULL;

  (*error_msg) = NULL;
  
  if ( lintar_get_prog_unzip()[0]=='\0' ) {

	(*error_msg) = g_strdup(_("ZIP archives not supported."));

  } else {

	/* Extract file into directory dir */
	if (selected_files!=NULL) {
	  l = g_list_first(selected_files);
	  argv = (gchar **)g_malloc( (gulong)(10+g_list_length(l))*sizeof(gchar*) );
	} else {
	  l = NULL;
	  argv = (gchar **)g_malloc( (gulong)(10)*sizeof(gchar*) );
	}
	
	i = 0;
	argv[i++] = g_strdup(lintar_get_prog_unzip()); 
	if (skip_new)  { argv[i++] = g_strdup("-u"); }
	if (overwrite) { argv[i++] = g_strdup("-o"); } else { argv[i++] = g_strdup("-n"); }
	argv[i++] = g_strdup("-d");
	argv[i++] = g_strdup(dir);
	argv[i++] = g_strdup(filename);
	
	while (l!=NULL) {
	  argv[i++] = g_strdup(l->data);
	  l = g_list_next(l);
	}
	argv[i] = NULL;
	
	/* Call tar to see contents of the file */
	spawn = thread_g_spawn_sync( NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, 
								 &standard_output, &standard_error, &exit_status,
								 &gerror);  //ignore errors

	g_strfreev(argv);

	if (standard_error!=NULL) {
	  if (standard_error[0]!='\000') {
		(*error_msg) = g_strdup(standard_error);
	  } 
	}
	
	if ( ((*error_msg)==NULL) && (exit_status!=0) ) {
	  (*error_msg) = g_strdup_printf(_("An error ocurred extracting file %s"), filename);
	}  
	if (spawn==FALSE) {
	  if (gerror->message!=NULL) {
		(*error_msg) = g_strdup(gerror->message);
	  } else {
		if ( (*error_msg)==NULL ) {
		  (*error_msg) = g_strdup_printf(_("An error ocurred extracting file %s"), filename);
		}
	  }
	}
	
	/* Free memory */
	if (standard_output!=NULL) {
	  g_free(standard_output);
	}
	if (standard_error!=NULL) {
	  g_free(standard_error);
	}

  }

}






